/*
 * eqcdplayer.C
 *
 * Copyright (C) 2000 Martin Pekar <cortex@post.sk>
 *
 * This program is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program; if not, write to the Free Software
 * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
 */


#include "eqcdplayer.h"

struct disc_info 	*the_info_a;
Fl_Counter		*track_counter;
Fl_Value_Output 	*label_minutes,	*label_seconds;

Fl_Button 		*button_play, *button_pause, *button_stop,
			*button_eject;
Fl_Slider 		*cdvolume_slider;
			
int 			cd_d;
int 			playing = 0; 
int			paused = 0; 
int 			track_to_play = 0; 
int 			last_tracks_number = 0;
int 			tracks_number = 0;

void playcb(Fl_Widget*,void*){
    playing = 1;
    if (track_to_play == 0) cd_play(cd_d,the_info_a->disc_first_track);
    cd_play(cd_d, track_to_play);
}



void stopcb(Fl_Widget *, void *data) {
    cd_stop(cd_d);    
    playing = 0;
}

void pausecb(Fl_Widget *, void *data) { 
    if (paused == 0) {
	cd_pause(cd_d);
	paused=1; 
    }
    else {
	cd_resume(cd_d); 
	paused=0; 
    }
}


void ejectcb(Fl_Widget *, void *data) { 
    cd_eject(cd_d);
}


void 
volume_changedcb(Fl_Widget *widget, void *data) {
    struct disc_volume volume;
    volume.vol_front.left = volume.vol_front.right = (int) widget->value();
    cd_set_volume(cd_d, volume);
}


void exitcb(Fl_Widget *, void *) { 
    exit(0); 
}

void about_cd_playercb(Fl_Widget*, void*) {
    fl_message("CD Player for Equinox Desktop Environment\n(c) 2000-2001, Martin Pekar\nv. 0.3\n");
}


void 
cb_setup_track(Fl_Counter* counter, void *track)
{
    track_to_play = (int) counter->value();
}



void 
anybody_out_there(void*) {

  the_info_a = new (struct disc_info);
  cd_d = cd_init_device("/dev/cdroms/cdrom0");  
  if (cd_d < 0 )  cd_d = cd_init_device("/dev/cdrom");  
  
  if (cd_d > 0 ) {
    
    cd_stat(cd_d, the_info_a);
    
    if (the_info_a->disc_present) { 
	
	int tracks_there = (int) the_info_a->disc_total_tracks;
	int some = 0;    
	for (int t = 1 ; t< tracks_there ; t++){
    	    if (the_info_a->disc_track[t].track_type == CDAUDIO_TRACK_AUDIO)  
		some++;
	    }
	    if ( some > 0 ) {
	    track_counter->maximum( (int) the_info_a->disc_total_tracks );
	    if ( playing == 1 ) {
	    label_minutes->value( the_info_a->disc_track_time.minutes );
    	    label_seconds->value( the_info_a->disc_track_time.seconds );
	    }
	    if ( playing == 0 ) {
	    label_minutes->value( 0 );
    	    label_seconds->value( 0 );
	    }

	    button_play->activate(); 
	    button_pause->activate();
    	    button_stop->activate(); 
	    //button_eject->activate();  //todo eject
	    cdvolume_slider->activate();
	    }
   	    else { // if data disk or what
    	    track_counter->maximum(0);
	    label_minutes->value(0);  
	    label_seconds->value(0);
	    button_play->deactivate(); 
	    button_pause->deactivate();
    	    button_stop->deactivate(); 
	    button_eject->deactivate();
	    cdvolume_slider->deactivate();
	    }
    
    }
    else {
    	track_counter->maximum(0); 
	label_minutes->value(0);
        label_seconds->value(0); 
	button_play->deactivate(); 
	button_pause->deactivate();
        button_stop->deactivate(); 
	button_eject->deactivate();
	cdvolume_slider->deactivate();

    }
    delete the_info_a;
  }
  
  if (cd_d < 0 ) {
  fl_alert("Neither /dev/cdrom nor /dev/cdroms/cdrom0 exist,\n please make symlink to your cdrom device!\n");
  exit(0);
  }
  Fl::add_timeout(1, anybody_out_there);    
}
  


int 
main(int argc, char ** argv) 
{

    Fl_Window* main_window = new Fl_Window(318, 131, "Equinox CD Player");
    Fl_Menu_Bar menubar(0, 0, 318, 25); 
        menubar.begin();
	    Fl_Item_Group file("&File");
    		Fl_Item quit_item("&Quit");
        	quit_item.shortcut(FL_ALT+'Q');
		quit_item.callback(exitcb);
	    file.end();
	    Fl_Item_Group help("&Help");
		Fl_Item about_item("&About");
        	about_item.shortcut(FL_ALT+'A');
     		about_item.callback(about_cd_playercb);
	    help.end();
    menubar.end();
		 
    button_play = new Fl_Button(25, 85, 60, 25, "&Play");
	button_play->callback((Fl_Callback*) playcb);          
	button_play->image( new Fl_Pixmap(play_xpm) );
	button_play->tooltip("Play music from CD");
        button_play->deactivate();
      
    button_pause = new Fl_Button(95, 85, 60, 25, "Pa&use");
	button_pause->callback((Fl_Callback*) pausecb);          
	button_pause->image( new Fl_Pixmap(pause_xpm) );
	button_pause->tooltip("Pause playing music");
        button_pause->deactivate();
    
    button_stop = new Fl_Button(165, 85, 60, 25, "&Stop");
        button_stop->callback((Fl_Callback*) stopcb);   
	button_stop->image( new Fl_Pixmap(stop_xpm) );       
	button_stop->tooltip("Stop playing music");
        button_stop->deactivate();
      
    button_eject = new Fl_Button(260, 90, 50, 20, "&Eject");
        button_eject->callback((Fl_Callback*) ejectcb);          
	button_eject->tooltip("Eject CD from CDROM (not functional for now)");
        button_eject->deactivate();
    
    cdvolume_slider = new Fl_Slider(5, 50, 75, 20, "Volume");
	  cdvolume_slider->type(5);
	  cdvolume_slider->tooltip("Set playing volume");
          cdvolume_slider->align(FL_ALIGN_TOP);
          cdvolume_slider->callback( (Fl_Callback*) volume_changedcb);          
	  cdvolume_slider->deactivate();
    
    track_counter = new Fl_Counter(245, 50, 70, 20, "Tracks:");
          track_counter->minimum(1);  
	  track_counter->maximum(1);
          track_counter->step(1);  
	  track_counter->value(1);
          track_counter->align(FL_ALIGN_TOP);
	  track_counter->tooltip("Choose track you want to play");
          track_counter->callback( (Fl_Callback*) cb_setup_track );          
     
    new Fl_Box(80, 35, 90, 10, "Elapsed time");
     label_minutes = new Fl_Value_Output(90, 45, 25, 25, "");
     new Fl_Box(115, 45, 15, 25, ":");
     label_seconds = new Fl_Value_Output(130, 45, 25, 25, "");
     
    main_window->end();
    main_window->show(argc,argv);

    Fl::add_timeout(1, anybody_out_there);

    return Fl::run();
}


