//============================================================================
//
//  ZZplayer port for Equinox Desktop Environment
//  Copyright (C) 2000 Martin Pekar <cortex@post.sk>
//
//  Requires Fast Light Tool Kit(FLTK)
//  http://www.agilinux.com/fltk
//	
//  Author  : Martin Pekar
//  E-Mail  : cortex@post.sk
//  Home    : http://www.cortex.host.sk

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================


#include "EPlayer.h"


void cb_play(Fl_Button*, void *);
void cb_stop(Fl_Button*, void*);
void cb_seek(Fl_Slider *vslider, void*);
void cb_sound(Fl_Widget*, void*);
void cb_loop(Fl_Widget*, void*);

void state_play();
void state_pause();
void state_stop();

void cb_set_fullscreen(Fl_Button*, void *);
void cb_volume_slider(Fl_Slider*vslider, void*);

void cb_open_file(Fl_Widget*, void*); 
void cb_open_vcd(Fl_Widget*, void*); 


void open_file();

void event_pool_timer(void*);
void update_info(void*);

void InfoDialog(Fl_Widget*, void*);
void about_player(Fl_Widget*, void*);
void cb_quit(Fl_Widget*, void*);

Plugin  		*genmpeg;

Zz_options 		option;
PluginManager 		*pluginmanager;
SMPEGState 		state = Stop;
char 			*vcd = "/dev/cdrom";
char 			*fileName = NULL;
char 			*clean_info = " Fps : 00.00 Frame : 00000\n File : No file";
char			*updated_info = " Fps : %.2f Frame : %.5i\n File : %s";

Fl_Item_Group 		*recent_items_group;

Fl_Button 		*button_play;
Fl_Button 		*button_stop;
//Fl_Button 		*button_fullscreen;
Fl_Button 		*turn_sound_button;
Fl_Button 		*turn_loop_button;

Fl_Slider		*volume_slider;
Fl_Slider		*seek_slider;
    

Fl_Box 			*info_label;

Fl_Window 		*main_window;
Fl_Window 		*button_bar;

class Video_Window : public Fl_Overlay_Window {
public:
  Video_Window(int x, int y, int w, int h) : Fl_Overlay_Window(x, y, w, h) {}
  void draw_overlay();
};

void Video_Window::
draw_overlay() {
  fl_color(FL_RED); 
  fl_rect(x(), y(), w(), h());
}

Video_Window 		*sub_window;


void
get_options() //todo
{
}


void 
cb_preferences(Fl_Widget*, void*) 
{
    Fl_Window *pref_window = new Fl_Window(300, 345, "Preferences");
    new Fl_Button(45, 310, 80, 25, "&OK");
    new Fl_Button(170, 310, 80, 25, "&Cancel");
    
    { Fl_Group* o = new Fl_Group(10, 25, 285, 50, "VCD");
      o->box(FL_ENGRAVED_BOX);
      o->align(FL_ALIGN_TOP | FL_ALIGN_LEFT);
      
      new Fl_Input(110, 40, 170, 20, "VCD Device:");
      
      o->end();
    }
    
    
    { Fl_Group* o = new Fl_Group(10, 105, 285, 110, "Options");
      o->box(FL_ENGRAVED_BOX);
      o->align(FL_ALIGN_TOP | FL_ALIGN_LEFT);
      
    new Fl_Check_Button(20, 110, 110, 25, "Bilinear filter");
    new Fl_Check_Button(20, 135, 110, 25, "Keep resolution");
    new Fl_Check_Button(20, 160, 110, 25, "Keep ratio");
    new Fl_Check_Button(20, 185, 180, 25, "Save options on exit");
    new Fl_Check_Button(165, 110, 100, 25, "Loop");
    new Fl_Check_Button(165, 135, 95, 25, "Auto close");
    
    o->end();
    }

    
    
    { Fl_Group* o = new Fl_Group(10, 235, 285, 55, "Sound");
      o->box(FL_ENGRAVED_BOX);
      o->align(FL_ALIGN_TOP | FL_ALIGN_LEFT);
      { Fl_Counter* o = new Fl_Counter(180, 260, 40, 15, "Startup volume");
        o->type(1);
        o->minimum(0);
        o->maximum(100);
        o->step(1);
        o->align(FL_ALIGN_TOP);
        o->tooltip("This volume will be set at startup");
      }
      
      new Fl_Check_Button(20, 240, 110, 25, "With audio");      
      
      o->end();
    }
    
    pref_window->end();
    pref_window->show();
}




//============================================================================

int 
eqdegui(int argc, char *argv[], int fullscreen, int volume, int autoclose, 
	    int loop, int keepthisresolution, int saveoptions, int windowID)
{
    
  main_window = new Fl_Window(300, 335, "ZZ Player for Equinox Desktop Environment");
  main_window->label_type(FL_NO_LABEL);
  
  Fl_Menu_Bar menubar(0, 0, 285, 25);
  menubar.begin();
	Fl_Item *about_menu_item = new Fl_Item();
        about_menu_item->image( new Fl_Pixmap(zzplayer_xpm) );
	about_menu_item->callback(about_player);
    Fl_Item_Group *file_group = new Fl_Item_Group("File");
          Fl_Item* open_file_item = new Fl_Item("Open file...");
        	   //open_item->image( new Fl_Pixmap(open_xpm) );
                   open_file_item->shortcut(FL_ALT + 'O');
		   open_file_item->callback(cb_open_file);

          Fl_Item* open_vcd_item = new Fl_Item("Open VCD...");
        	   //open_item->image( new Fl_Pixmap(open_xpm) );
                   open_vcd_item->shortcut(FL_ALT + 'V');
		   open_vcd_item->callback(cb_open_vcd);
		   
 	  Fl_Item *close_item = new Fl_Item("Close");
                   close_item->shortcut(FL_ALT + 'C');
		   //close_item->callback(cb_close_file);
		   close_item->callback( (Fl_Callback*) cb_stop );
		   
		   
 	  Fl_Item *options_item = new Fl_Item("Preferences...");
                   options_item->shortcut(FL_ALT + 'R');
		   //options_item->callback((Fl_Callback*) cb_preferences);
		   
      	  new Fl_Divider();
	  
          Fl_Item* quit_item = new Fl_Item("Quit");
                   quit_item->shortcut(FL_ALT + 'Q');
		   quit_item->callback(cb_quit);
      file_group->end();
      
      Fl_Item_Group options_group("Options");
          Fl_Item* play_pause_item = new Fl_Item("Play/Pause");
                   play_pause_item->shortcut(FL_ALT + 'P');
		   play_pause_item->callback( (Fl_Callback*) cb_play );
	  new Fl_Divider();		   
          Fl_Item* stop_item = new Fl_Item("Stop");
                   stop_item->shortcut(FL_ALT + 'S');
		   stop_item->callback( (Fl_Callback*) cb_stop );
          Fl_Item* loop_item = new Fl_Item("Loop on/off");
                   loop_item->shortcut(FL_ALT + 'L');
		   loop_item->callback( (Fl_Callback*) cb_loop);
          Fl_Item* sound_item = new Fl_Item("Sound on/off");
                   sound_item->shortcut(FL_ALT + 'D');
		   sound_item->callback( (Fl_Callback*) cb_sound);
	  new Fl_Divider();		   
	  Fl_Item* info_item = new Fl_Item("File info...");
                   info_item->shortcut(FL_ALT + 'I');
		   info_item->callback( (Fl_Callback*) InfoDialog);

	  new Fl_Divider();		   
          recent_items_group = new Fl_Item_Group("Recent files");
          file_group->end();

      options_group.end();
      
      Fl_Item_Group help_group("Help");
	  Fl_Item* about_item = new Fl_Item("About");
	           about_item->image( new Fl_Pixmap(zzplayer_xpm) );
                   about_item->shortcut(FL_ALT + 'A');
		   about_item->callback(about_player);
      help_group.end();
  menubar.end();


    button_bar = new Fl_Window(5, 230, 290, 110);
    button_bar->begin();

    button_play = new Fl_Button(35, 75, 40, 25);      
    button_play->image( new Fl_Pixmap(play_xpm) );
    button_play->tooltip("Play selected file");
    button_play->callback( (Fl_Callback*) cb_play, 0 );
      
    button_stop = new Fl_Button(80, 75, 40, 25);
    button_stop->image( new Fl_Pixmap(stop_xpm) );
    button_stop->tooltip("Stop selected file");
    button_stop->callback( (Fl_Callback*) cb_stop, 0 );

//    button_fullscreen = new Fl_Button(145, 75, 25, 25);
//    button_fullscreen->callback( (Fl_Callback*) cb_set_fullscreen );
//    button_fullscreen->image( new Fl_Pixmap(fullscreen_xpm) );
//    button_fullscreen->tooltip("Turn on fullscreen mode");

    volume_slider = new Fl_Slider(245, 5, 15, 45);
    volume_slider->type(4);
    volume_slider->tooltip("Set volume");
    volume_slider->maximum(100);
    volume_slider->step(1);
    volume_slider->value(80);
    volume_slider->callback( (Fl_Callback*) cb_volume_slider);

      
    seek_slider = new Fl_Slider(35, 55, 165, 15);
    seek_slider->type(5);
    seek_slider->image( new Fl_Pixmap(video_xpm) );
    seek_slider->align(FL_ALIGN_LEFT);
    seek_slider->tooltip("Seek through file");
    seek_slider->maximum(100);
    seek_slider->step(1);
    seek_slider->callback( (Fl_Callback*) cb_seek);

    info_label = new Fl_Box(35, 5, 165, 45, clean_info);
    info_label->tooltip(clean_info);		    
    info_label->align(FL_ALIGN_LEFT|FL_ALIGN_INSIDE);
    info_label->box(FL_ENGRAVED_BOX);
    info_label->color(FL_BLACK);
    info_label->label_color(FL_GREEN);
    
    turn_sound_button = new Fl_Button(215, 15, 25, 25);
    turn_sound_button->image( new Fl_Pixmap(sound_xpm) ); 
    turn_sound_button->tooltip("Turn sound on/off");		    
    turn_sound_button->callback( (Fl_Callback*) cb_sound);

    turn_loop_button = new Fl_Button(175, 75, 25, 25);
    turn_loop_button->image( new Fl_Pixmap(loop_xpm) );
    turn_loop_button->tooltip("Turn loop on/off");		    
    turn_loop_button->align(FL_ALIGN_CENTER|FL_ALIGN_INSIDE);
    turn_loop_button->callback( (Fl_Callback*) cb_loop);

    Fl_Button *configure_button = new Fl_Button(220, 75, 25, 25);
    configure_button->image( new Fl_Pixmap(configure_xpm) );
    configure_button->tooltip("Configure player preferences");		    
    
    Fl_Button *help_button = new Fl_Button(245, 75, 25, 25);
    help_button->image( new Fl_Pixmap(help_xpm) );
    help_button->tooltip("Help for usage");		    
    
    button_bar->end();
    

    
    //sub_window = new Fl_Window(5, 25, 290, 200);
    sub_window = new Video_Window(5, 25, 290, 200);
    sub_window->begin();
    sub_window->color(FL_BLACK);
    sub_window->end();

    main_window->end();
    //main_window->resizable(main_window);
    main_window->show(1, argv);
    sub_window->show_inside(main_window);
    sub_window->show_inside(button_bar);



    // default initial staff
    option.windowId = (int) fl_xid(sub_window);
    option.keepResolution = 1;
    option.keepRatio = 1;
    option.bilinearFilter = 1;
    option.VCDDevice = "/dev/cdrom";
    option.closeOnEnd = 0;
    option.endLessLoop = 0;
    option.saveOnExit = 0;
    option.withAudio = 1;


    pluginmanager = new PluginManager();
	
    // todo from config initial staff

    // from command line initial staff
//    if (fullscreen !=2){
//		option.fullScreenOnOpen = (fullscreen != 0);
//	}
	if (autoclose !=2){
		option.closeOnEnd = (autoclose != 0);
	}
	if (loop !=2){
		option.endLessLoop = (loop != 0);
	}
	if (keepthisresolution !=2){
		option.keepResolution = (keepthisresolution != 0);
	}
	if (saveoptions !=2){
		option.saveOnExit = (saveoptions != 0);
	}

	if (volume!=-2){
	  if (volume >= 0 & volume <= 100){
	    volume_slider->value(volume);
	    option.withAudio = 1;
	  }
	  else {
	    option.withAudio = 0;
          }
        }

	turn_loop_button->value(option.endLessLoop);
	turn_sound_button->value(option.withAudio);
	volume_slider->activate(option.withAudio);

    Fl::add_timeout(1, update_info);    
    
    if (argc > 1) {
//	fileName = argv[1];
//        open_file();
    }
    
    return Fl::run();
}


void 
apply_config() //todo
{
}

void
cb_sound(Fl_Widget*, void*)
{
	option.withAudio = !option.withAudio;
	volume_slider->activate(option.withAudio);
	if ( (genmpeg != NULL) && genmpeg->hasSound() ) {                                    
  	    genmpeg->setVolume(option.withAudio ? (int) volume_slider->value() : 0);
	}
	turn_sound_button->value(option.withAudio);
}

void
cb_loop(Fl_Widget*, void*)
{
    option.endLessLoop = !option.endLessLoop;
    turn_loop_button->value(option.endLessLoop);
}

void 
update_info(void*)
{
//    printf("update info\n");

    if ( (genmpeg != NULL) || (state == Play) ) {
        int seek, frame;
        double fps;

        genmpeg->getInfo(&seek, &fps, &frame);
        char *label = new char[80];	
	char *file = strrchr(fileName, '/');
        snprintf(label, 80, updated_info, fps, frame, ++file); 
        info_label->label(label);
	info_label->tooltip(label);
        info_label->redraw();
        seek_slider->value(seek);	
	
	if ( state != Pause) {
	    if  (genmpeg->status() == STREAM_STOPPED ) {
		state_stop();
	     }
	}
    }
    
    Fl::add_timeout(2, update_info);    
}


void 
cb_seek(Fl_Slider *slider, void*)
{
    if (genmpeg != NULL)  
	genmpeg->setSeek( (int) seek_slider->value() ); 
}



void 
event_pool_timer(void*) //todo
{
/*    if (genmpeg == NULL) 
	return;
    genmpeg->videoEvents();
    if (genmpeg->quitEvent()) {
     //close();
    }
    
    Fl::add_timeout(1, event_pool_timer);    */
}


void
state_play()
{
    button_play->image( new Fl_Pixmap(pause_xpm) );
    button_play->redraw();
    state = Play;
}

void
state_pause()
{
    button_play->image( new Fl_Pixmap(play_xpm) );
    button_play->redraw();
    state = Pause;
}


void
state_stop()
{
//    printf("state stop\n");
    seek_slider->value(0);	
    sub_window->redraw();
    info_label->label(clean_info);
    info_label->redraw();
    info_label->tooltip(clean_info);		    
    button_play->image( new Fl_Pixmap(play_xpm) );
    button_play->redraw();
    state = Stop;
}



void 
cb_play(Fl_Button*, void *) 
{
    if ( (genmpeg == NULL) || (state == Stop) ) {
	cb_open_file(0, 0);
	return;
    }
    
    genmpeg->pause();
    
    if (state == Play) {	
	state_pause();
    }
    else {
	state_play();
    }
    
}


void 
cb_volume_slider(Fl_Slider *vslider, void *) 
{
    if(genmpeg == NULL)	
	return;	
    genmpeg->setVolume( (int) volume_slider->value() );
}


void
close_file()
{
    if (genmpeg != NULL) { 
	delete genmpeg;
	genmpeg = NULL;
    }
}

void 
cb_stop(Fl_Button *, void *) 
{
    state_stop();
    close_file();
}


void 
cb_open_file(Fl_Widget *, void *) 
{
//    printf("cb open file\n");
    char *f = fl_file_chooser("Open Audio/Video file ...", "*.mp*", "/" );
    if (f) {
	fileName = f;
	open_file();
    }
}


void 
cb_open_vcd(Fl_Widget*, void*)
{
    fileName  = new char[80];
    snprintf(fileName, 80, "vcd:%s", vcd);
    open_file();
} 

void 
OpenLastFile(Fl_Widget *, char *lastFile )
{
    fileName  = lastFile;
    open_file();
}

void 
open_file() 
{
//    printf("open file\n");
        
    state_stop();
    close_file();
    
    int volume = option.withAudio ? (int) volume_slider->value() : 0;

    genmpeg = pluginmanager->openStream(fileName, option.withAudio, 1, option.windowId, NULL);
    
    if ( genmpeg->error() ) {
	fl_message("ZZ Player: %s\nCheck the file your try to open,\nor device (/dev/cdrom[default]) for VCD", genmpeg->error());
        state_stop();
	delete genmpeg;
	genmpeg = NULL;
    }

    else {
        recent_items_group->begin();
	char *fileNameLast = new char[strlen(fileName) + 1];
	strcpy(fileNameLast, fileName);
	
    	Fl_Item *last_item = new Fl_Item(fileNameLast);
	last_item->callback( (Fl_Callback*) OpenLastFile, fileNameLast);
	recent_items_group->end();
	
	state_play();
	genmpeg->setVolume(volume);
        genmpeg->setBilinearFilter(option.bilinearFilter);
        genmpeg->setKeepThisResolution(option.keepResolution);
        genmpeg->setKeepRatio(option.keepRatio);
	
	StreamInfo info;
        genmpeg->getFullInfo(&info);
  
	if ( info.has_video ) {
	    int video_width = info.width; 
	    int video_height = info.height;
	    
	    if (info.width < 290) video_width = 290;
	    if (info.height < 200) video_height = 200;

	    main_window->size(video_width + 10, video_height + 25 + 5 + 105);
	    sub_window->size(video_width, video_height);
	    button_bar->move(button_bar->x(), video_height + 25 + 5 );
	} //if
    }//else
    
	
} 


void 
InfoDialog(Fl_Widget*, void*)
{

    if (genmpeg != NULL) {
	char *stream = "None";
	char *audio = "None";
	int size = 0;
	int videoWidth = 0; 
	int videoHeight = 0;
        StreamInfo info;
	
        genmpeg->getFullInfo(&info);

        if ( info.has_audio && info.has_video ) {
	    stream = "MPEG system stream (audio/video)";
        } 
        else if ( info.has_audio ) {
	    stream = "MPEG audio stream only";
        } 
        else if ( info.has_video ) {
	    stream = "MPEG video stream only";
        }
  
	if ( info.has_video ) {
	    videoWidth = info.width; 
	    videoHeight = info.height;
        }    

	if ( info.has_audio ) {
	    audio = info.audio_string;
	}
	
	if ( info.total_size ) {
	    size = info.total_size;
	}
	fl_message("Stream: %s\nVideo Width: %d\nVideo Height: %d\nAudio: %s\nFile size: %d",
	    stream, videoWidth, videoHeight, audio, size);	
    }
}



void 
cb_quit(Fl_Widget*, void*)
{
    exit(0);
}


void 
about_player(Fl_Widget*, void*) 
{
    fl_message("ZZ Player port for Equinox Desktop Environment\n(c) 2000-2001, Martin Pekar\n");
}


