/*  gmpeg- MPEG-1 player for Linux
 *
 *  Copyright (C) 2000 Jayan. S. R
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation; either version 2 of the License, or
 *  (at your option) any later version.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You should have received a copy of the GNU General Public License
 *  along with this program; if not, write to the Free Software
 *  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
 */
#include "gmpeg.h"

extern int loop_active;
extern int zoom_active;


SMPEG *mpeg;
SMPEG_Info info;
SDL_Surface *sdl_screen;
SDL_Event event;
int enable_audio=1;

int sdl_initialize ()
{
  if ( SDL_Init (SDL_INIT_VIDEO) < 0 ) 
	{
		fprintf ( stderr, "Unable to initialize SDL video %s\n", SDL_GetError() );
		return -1;
	}
  if ( SDL_Init ( SDL_INIT_AUDIO ) < 0 )
	{
	  fprintf ( stderr, "Unable to initialize SDL audio %s\n", SDL_GetError() );
		enable_audio = 0;
		return -1;
	}

	atexit ( SDL_Quit );

  return 1;
}

int play_mpeg( char *file_name )
{
	
	char message[1024];
	mpeg = SMPEG_new ( file_name, &info, 1 );
	
	if ( SMPEG_error( mpeg ) ) {
		printf ( "smpeg error %s\n", SMPEG_error( mpeg ) );
		sprintf( message, SMPEG_error( mpeg) );
		dialog_box_error (message, "Okay" ); 
		return 0;
	}
	
	SMPEG_enableaudio ( mpeg, enable_audio );
	
	if ( sdl_screen )
	{
		SDL_FreeSurface ( sdl_screen );
		sdl_screen = NULL;
	}

	if ( info.has_video )
	{
		sdl_screen = SDL_SetVideoMode ( info.width, info.height,0 , SDL_HWSURFACE );	}
	
	SDL_WM_SetCaption( file_name, "gmpeg video: ");
	SMPEG_setdisplay ( mpeg, sdl_screen, NULL, NULL  );
	
	SMPEG_getinfo ( mpeg, &info );

	if ( mpeg ) {
		if ( enable_audio == 1 )
			SMPEG_enableaudio( mpeg, 1 ); /* check this code */
		SMPEG_rewind ( mpeg );
		SMPEG_play ( mpeg );
	}
	return 0;
}

void pause_mpeg ( )
{
	int status=2;
	if ( mpeg ) {
		status = SMPEG_status ( mpeg );
		if ( status== 0|1 )              /* SMPEG_STOPPED = 0 */
    //if ( status== SMPEG_STOPPED | SMPEG_PLAYING )
		  SMPEG_pause ( mpeg );          /* SMPEG_PLAYING = 1 */
	}
	g_print ( "status: %d\n", status );
}

void stop_mpeg ( )
{
	int status;

	if ( mpeg ) {
		status = SMPEG_status( mpeg );
		if ( status==SMPEG_PLAYING ) {
			SMPEG_stop ( mpeg );
			SMPEG_rewind ( mpeg );
		}
	  g_print ( "status: %d\n", status );
	}
}


void resume_mpeg ( ) /* resume_mpeg selection of name is ****ed up */
{										 /* does not mean what it does */
	int status;
	
	if ( mpeg ) 
	{
		status = SMPEG_status( mpeg );
		if ( status==SMPEG_STOPPED ) 
		{
			SMPEG_rewind ( mpeg );
			SMPEG_play ( mpeg );
		}
	}
}

void loop_mpeg ( )
{
	if ( mpeg )
	{
		if ( loop_active ) 
		{
			SMPEG_loop ( mpeg, 1 );
		}
		else 
		{
			SMPEG_loop ( mpeg, 0 );
		}
	}
}

void zoom_mpeg ( )
{
	int status;

	if ( !mpeg ) return;

	status = SMPEG_status ( mpeg );

	if ( status == 1 )
	{
		SMPEG_stop( mpeg );
		g_print ( "mpeg stopped\n");
	}

	if ( sdl_screen ) 
	{
		SDL_FreeSurface ( sdl_screen );
		sdl_screen = NULL;
	}

	if ( info.has_video )
	{
		if ( zoom_active == 1 ) 
		{
			sdl_screen = SDL_SetVideoMode ( info.width*2, info.height*2,0 , 
		                                SDL_HWSURFACE );	
		}
		else
		{
			sdl_screen = SDL_SetVideoMode ( info.width, info.height, 0, 
			                                SDL_HWSURFACE );
		}
	}

	if ( mpeg && sdl_screen )
	{
		SMPEG_double ( mpeg, zoom_active );
		SMPEG_play ( mpeg );
	}
	
}

void set_volume_mpeg ( int volume )
{
	
	if ( mpeg )
	{
		SMPEG_setvolume( mpeg, volume );
	}
}

void full_screen_mpeg ( )
{
	if ( sdl_screen )
	{
  	SDL_Event event;
  	SDL_WM_ToggleFullScreen( sdl_screen );
  
  	SDL_ShowCursor ( 0 );
  
    SDL_EventState (SDL_ACTIVEEVENT, SDL_IGNORE);
    SDL_EventState (SDL_MOUSEMOTION, SDL_IGNORE);
    while (!SDL_PollEvent (&event) && SMPEG_status (mpeg) == SMPEG_PLAYING)
      SDL_Delay(100);  
	

  	SDL_WM_ToggleFullScreen(sdl_screen);

    SDL_ShowCursor(1);
	}
}

void move_frames_mpeg ( int position, GtkAdjustment *adj  )
{
	if ( ( mpeg ) && ( SMPEG_PLAYING ) )
	{
   SMPEG_seek ( mpeg, (position*info.total_size/100 ) );  
	}
	else 
	{
		gtk_adjustment_set_value ( adj, 0.0 );
	}
}

int is_mpeg_playing ()
{
  if ( mpeg && SMPEG_PLAYING )
	  return 1;
	else 
	  return 0;
}

Uint32 get_position_of_mpeg ()
{
	if ( mpeg && (SMPEG_status(mpeg)==SMPEG_PLAYING) )
	{
	  SMPEG_getinfo( mpeg, &info );
		return info.current_offset;
	}
	else return 0;
}

Uint32 get_total_size_of_mpeg ( )
{
	if ( mpeg )
		return info.total_size;
	return 0;
}

void skip_frames_mpeg ( int ffrw )
{
	int position;
	int total;
	int num_frames_fastfw;
	int num_frames_rewind;
	if ( mpeg && ( SMPEG_status( mpeg)==SMPEG_PLAYING ) )
	{
	  if ( ffrw == FASTFW )  
	  {
		  SMPEG_getinfo( mpeg, &info );
			position = info.current_offset;
			total = info.total_size;
			num_frames_fastfw = total/10;
			g_print( "frames fastfw = %d\n", num_frames_fastfw );
			g_print( "total = %d\n", total );
			if ( ( position + num_frames_fastfw ) < total )
        SMPEG_seek ( mpeg, (position + num_frames_fastfw ) );  
	  }
	  else  
	  {
	    SMPEG_getinfo( mpeg, &info );
			position = info.current_offset;
			total = info.total_size;
			num_frames_rewind = total/10;
			if ( ( position - num_frames_rewind) )
			  SMPEG_seek ( mpeg, (position -num_frames_rewind) );
		}
	}
}
