//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//  Home    : http://www.chez.com/tsc/zzplayer/zzplayer.html
//
//	Requires the Qt widget libraries, available at no cost at
//      http://www.troll.no
//
//  File    : playlist.cpp
//  Author  : Frederic Grasset
//  E-Mail  : frederic.grasset@wanadoo.fr

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================


#include "ZZlist.h"
#include "playlist.h"
#include "playlist.moc"


ZZplaylist::ZZplaylist()
:QWidget()
{
	DEBUGMSG("ZZplaylist::ZZplaylist(...)");

	setCaption(i18n("ZZplayer : Playlist"));
	
	list.setAutoDelete( true );

	currentpos=-1;

//***
//* Construct the gui
//***

// We organize our widgets by line
	QBoxLayout *topLayout = new QVBoxLayout( this, 6 );

// 1st widget's line : title of playlist
// A QComboBox that permit chosing a collection of recent list
//	titre = new QComboBox(this);
//	FIXE_HAUT(titre);
//	topLayout->addWidget(titre);


// 2nd widget's line : The list with the controls
// A QHBoxLayout
	QBoxLayout *layout1 = new QHBoxLayout( 6 ); // yes, we loose previous widget ref!
	topLayout->addLayout(layout1);

	// 1st widget's column : the list
	// A QListBox thet display the current playlist
	pList = new QListBox ( this );
//	pList->setFrameStyle( QFrame::Panel | QFrame::Sunken );
//	pList->setLineWidth( 2 );
	pList->setMinimumWidth(150);
	connect(pList, SIGNAL(selected(int)),SLOT(slotSelectFile(int)));
	layout1->addWidget(pList,2);

	// 2nd widget's column : the controls
	// A QVBoxLayout
	QBoxLayout *layout2 = new QVBoxLayout( 12);
	layout1->addLayout(layout2);

		// 1st widget's line : playlist's controls
		// A  QGroupBox
	QGroupBox *box = new QGroupBox(i18n("Playlist"),this);
	layout2->addWidget(box,0,AlignRight);

	QBoxLayout *layout3 = new QVBoxLayout( box, 6 );

	layout3->addSpacing( box->fontMetrics().height() );

	QPushButton *widget = new QPushButton(i18n("&New"),box);
	FIXE_MIN(widget);
	layout3->addWidget(widget);
	connect(widget,SIGNAL(clicked()),SLOT(slotNewList()));

	widget = new QPushButton(i18n("&Load"),box); // yes, we loose previous widget ref!
	FIXE_MIN(widget);
	layout3->addWidget(widget);
	connect(widget,SIGNAL(clicked()),SLOT(slotLoadList()));

	widget = new QPushButton(i18n("&Save"),box); // yes, we loose previous widget ref!
	FIXE_MIN(widget);
	layout3->addWidget(widget);
	connect(widget,SIGNAL(clicked()),SLOT(slotSaveList()));

	layout3->activate();

		// 2nd widget's line : playlist's controls
		// A  QGroupBox
	box = new QGroupBox(i18n("File"),this); // yes, we loose previous widget ref!
	layout2->addWidget(box,0,AlignRight);

	layout3 = new QVBoxLayout( box, 6 ); // yes, we re-use this name too!

	layout3->addSpacing( box->fontMetrics().height() );

	widget = new QPushButton(i18n("&Add"),box); // yes, we loose previous widget ref!
	FIXE_MIN(widget);
	layout3->addWidget(widget,1);
	connect(widget,SIGNAL(clicked()),SLOT(slotAddFile()));

	widget = new QPushButton(i18n("&Remove"),box); // yes, we loose previous widget ref!
	FIXE_MIN(widget);
	layout3->addWidget(widget,1);
	connect(widget,SIGNAL(clicked()),SLOT(slotRmFile()));

	layout3->activate();

		// 3th widget's line : sorting controls
		// A  QGroupBox
	box = new QGroupBox(i18n("Sort"),this); // yes, we loose previous widget ref!
	layout2->addWidget(box,0,AlignRight);

	layout3 = new QVBoxLayout( box, 6 ); // yes, we re-use this name too!

	layout3->addSpacing( box->fontMetrics().height() );

	widget = new QPushButton(i18n("&Up"),box); // yes, we loose previous widget ref!
	FIXE_MIN(widget);
	layout3->addWidget(widget,1);
	connect(widget,SIGNAL(clicked()),SLOT(slotMoveUp()));

	widget = new QPushButton(i18n("&Down"),box); // yes, we loose previous widget ref!
	FIXE_MIN(widget);
	layout3->addWidget(widget,1);
	connect(widget,SIGNAL(clicked()),SLOT(slotMoveDown()));

	layout3->activate();


	layout2->addStretch(10);

		// last widget's line : close button
		// A QPushButton
	QPushButton *close = new QPushButton(i18n("&Close"),this);
	FIXE_TAILLE(close);
	layout2->addWidget(close,0,AlignRight);
	connect(close,SIGNAL(clicked()),SLOT(slotClosePlaylist()));

//***
// non GUI QT object
//***
	KDNDDropZone * dropZonePL = new KDNDDropZone( this, DndURL );
	connect( dropZonePL, SIGNAL( dropAction( KDNDDropZone *) ), SLOT( slotDropAction( KDNDDropZone *) ) );


// We finish this tab, we can activate layout management
	topLayout->activate();
}

ZZplaylist::~ZZplaylist()
{
	DEBUGMSG("ZZplaylist::~ZZplaylist(...)");

}

/****
** methodes definitions
****/

/**
* publics methods
**/
bool ZZplaylist::addList( QStrList &lst, bool rec, bool newList)
{
	DEBUGMSG("ZZplaylist::addFile(...)");

	if (newList)
		{
		list.clear();
		pList->clear();
		setCurrent(-1);
		}
	for (char *str=lst.first(); str != NULL; str=lst.next())
	  {
	  QString qstr(str);
	  addString(qstr);
	  }
  	if (currentpos < 0)
	  setCurrent(0);
	return true;
}

bool ZZplaylist::addFile( QString &file, bool rec, bool newList )
{
	DEBUGMSG("ZZplaylist::addFile(...)");

	if (newList)
		{
		list.clear();
		pList->clear();
		setCurrent(-1);
		}
  	addString(file);
  	if (currentpos < 0)
		setCurrent(0);
	return true;
}

QString *ZZplaylist::firstFile()
{
	DEBUGMSG("ZZplaylist::firstFile(...)");

	QString *res=list.getFirst();
	if (res != NULL)
		setCurrent(0);
	return res;
}

QString *ZZplaylist::lastFile()
{
	DEBUGMSG("ZZplaylist::lastFile(...)");

	QString *res=list.getLast();
	if (res != NULL)
		setCurrent(list.count()-1);
	return res;
}

QString *ZZplaylist::nextFile()
{
	DEBUGMSG("ZZplaylist::nextFile(...)");

	if (currentpos < 0)
		return NULL;
  	int npos=currentpos+1;
  	QString *res=list.at(npos);
	if (res != NULL)
		setCurrent(npos);
	return res;
}

QString *ZZplaylist::prevFile()
{
	DEBUGMSG("ZZplaylist::prevFile(...)");

	if (currentpos < 0)
		return NULL;
  	int npos=currentpos-1;
  	QString *res=list.at(npos);
	if (res != NULL)
		setCurrent(npos);
	return res;
}

QString *ZZplaylist::currentFile()
{
	DEBUGMSG("ZZplaylist::currentFile(...)");

	return list.at(currentpos);
}


/**
* privates methods
**/

void ZZplaylist::addString(QString &str)
{
  int pos=str.find("file:");
  if (pos == 0)
	{
	str.remove(0,5);
	}
  list.append(new QString(str));
  QString *tmp=get_file_name(str);
  pList->insertItem(new  ZZlistItem(*tmp));
  delete tmp;
}

// *** Warning : this function assume p1 and p2 are valid indexes ***
void ZZplaylist::swapFile(int p1, int p2)
{
  int h,b;
  ZZlistItem *i1,*i2;
  QString *st1,*st2;

  if (p1 < p2)
	{
	h=p1;
	b=p2;
	}
  else
	{
	h=p2;
	b=p1;
	}
  st1=list.take(b);
  st2=list.take(h);
  list.insert(h,st1);
  list.insert(b,st2);
  i1=new ZZlistItem(pList->text(h));
  i2=new ZZlistItem(pList->text(b));
  if (currentpos == h)
	{
	i1->setActive();
	currentpos=b;
	}
  else if (currentpos == b)
	{
	i2->setActive();
	currentpos=h;
	}
  pList->changeItem(i1,b);
  pList->changeItem(i2,h);
}

QString *ZZplaylist::get_file_name(QString &file)
{
	DEBUGMSG("ZZplaylist::get_file_name(...)");

	int pos=file.findRev('/');
	if (pos < 0)
		return new QString(file);
	return new QString(file);

}

void ZZplaylist::setCurrent(int pos)
{
  ZZlistItem *item;

  if (currentpos >= 0)
	{
	  item=new ZZlistItem(pList->text(currentpos));
	  pList->changeItem(item,currentpos);
	}
  currentpos=pos;
  if (pos < 0)
	return;
  item=new ZZlistItem(pList->text(pos));
  item->setActive();
  pList->changeItem(item,pos);
}

/****
** Slots definitions
****/


void ZZplaylist::slotNewList()
{
 	DEBUGMSG("ZZplaylist::slotNewList(...)");

	list.clear();
	pList->clear();
	currentpos=-1;
	emit removeCurrentTitle();
}

void ZZplaylist::slotLoadList()
{
 	DEBUGMSG("ZZplaylist::slotLoadList(...)");

	QString fileName;
	fileName=KFileDialog::getOpenFileName(fileName,	"*|All Files (*)\n");
	if (fileName.isEmpty())
		return;
	QFile file(fileName);
	if (!file.open(IO_ReadOnly))
		{
		KMsgBox::message( this, fileName, i18n("Can't open this file in read mode."));
		return;
		}
	QTextStream s(&file);
	QString line;
	while (!s.eof())
		{
		line=s.readLine();
		addString(line);
		}
	file.close();
	setCurrent(0);
	
}

void ZZplaylist::slotSaveList()
{
 	DEBUGMSG("ZZplaylist::slotSaveList(...)");

	QString fileName;
	fileName=KFileDialog::getOpenFileName(fileName,	"*|All Files (*)\n");
	if (fileName.isEmpty())
		return;
	QFile file(fileName);
	if (!file.open(IO_WriteOnly))
		{
		KMsgBox::message( this, fileName, i18n("Can't open this file in write mode."));
		return;
		}
	QTextStream s(&file);
	for (QString *line=list.first(); line != NULL ; line=list.next())
		{
		s << *line << endl;
		}
	file.close();
}

void ZZplaylist::slotAddList()
{
	DEBUGMSG("ZZplaylist::slotAddList(...)");

}

void ZZplaylist::slotAddFile()
{
	DEBUGMSG("ZZplaylist::slotAddFile(...)");

	QString fileName;
	fileName=KFileDialog::getOpenFileName(fileName,	"*.mpg *.mpeg *.mpe *.MPG *.MPEG *.MPE *.MP3 *.mp3|Mpeg Files\n"
							"*|All Files (*)\n");
	addFile(fileName);
}

void ZZplaylist::slotRmFile()
{
	DEBUGMSG("ZZplaylist::slotRmFile(...)");

	int index=pList->currentItem();
	if (index < 0)
		return;
  	pList->removeItem(index);
	list.remove(index);
	bool sig=(index == currentpos);
	if (index >= currentpos)
		setCurrent(currentpos-1);
  	if (sig)
		emit removeCurrentTitle();
}

void ZZplaylist::slotMoveUp()
{
  int pos=pList->currentItem();

  if (pos <= 0)
	return;
  swapFile(pos,pos-1);
  pList->setSelected(pos-1,true);
}

void ZZplaylist::slotMoveDown()
{
  int pos=pList->currentItem();
  
  if (pos >= pList->count()-1)
	return;
  swapFile(pos,pos+1);
  pList->setSelected(pos+1,true);
}

void ZZplaylist::slotClosePlaylist()
{
	DEBUGMSG("ZZplaylist::slotClosePlaylist(...)");

	hide();
}

void ZZplaylist::slotSelectFile(int index)
{
	setCurrent(index);
	emit selectTitle();
}

void ZZplaylist::slotDropAction(KDNDDropZone *url)
{
  emit dropAction(url);
}
