//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//
//	Requires the Qt widget libraries, available at no cost at
//      http://www.troll.no
//	
//  Author  : Nicolas Vignal
//  E-Mail  : nicolas.vignal@fnac.net
//  Home    : http://zzplayer.sourceforge.net

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#include "zzplayer.h"
#include "zzplayer.moc"
#define extraSpace 1
#define IDA 3 // why a so mysterious name? so keep the secret ;-)

int kdeCallback( ZZ_EventStruct *event)
{
 	DEBUGMSG("ZZplayer::kdeCallback(...)");

  if (event->type==ZZ_MOUSEBUTTONDOWN)
	{
  	if ((char)event->data==1)
	  { // left button
  	  player->fscreen=!player->fscreen;
  	  player->genmpeg->fullScreen(player->fscreen);
	  }
  	else if ((char)event->data==3)
	  { // right button
	  player->slotPlay();
	  }
  	else if ((char)event->data==MOUSEBUTTON_SCROLLUP)
	  {
	  player->increaseSound(5);
	  }
  	else if ((char)event->data==MOUSEBUTTON_SCROLLDOWN)
	  {
	  player->decreaseSound(5);
	  }
	}
  else if (event->type==ZZ_QUIT)
	{
	player->close();
	}
  else
	{
	DEBUGMSG("event : " << event->type);
	}
	
  return 0;

}


static const char labelTemplate[]=" Fps : 0.00\n Frame : 00000";

//Constructeur
ZZplayer::ZZplayer(QString file, int fullscreen, int volume, int autoclose , int loop, int keepthisresolution, int saveoptions, int windowID)
:QWidget(0,0,0), prevPix(ICON(PIXMAP_PATH"/prevtrack.xpm")),playPix(ICON(PIXMAP_PATH"/play.xpm")), nextPix(ICON(PIXMAP_PATH"/nexttrack.xpm")), pausePix(ICON(PIXMAP_PATH"/pause.xpm")) ,
soundPix(ICON(PIXMAP_PATH"/sound.xpm")), nosoundPix(ICON(PIXMAP_PATH"/nosound.xpm"))
{
	DEBUGMSG("ZZplayer::ZZplayer(...)");

	setCaption(i18n("ZZplayer"));

	KStdAccel stdAccel;

	// Setup
	plWindow= new ZZplaylist();
	if (plWindow == NULL)
		exit(1);

	pluginmanager = new PluginManager();
	if (pluginmanager == NULL)
	  {
	  exit (1);
	  }

  	connect(plWindow,SIGNAL(removeCurrentTitle()),SLOT(slotFileChange()));
  	connect(plWindow,SIGNAL(selectTitle()),SLOT(slotStop()));
  	connect(plWindow,SIGNAL(selectTitle()),SLOT(slotPlay()));
	connect(plWindow,SIGNAL(dropAction(KDNDDropZone *)),SLOT(slotDropAction(KDNDDropZone *)));
	genmpeg=NULL;
	state=Stop;
	
// ToDo : ZZoptions must containt some code for init itself!
	option.windowId=windowID;
	option.VCDDevice="/dev/cdrom";

	optionBox=NULL;

	// Control panel
	createControl();

	// Read the config file
	readOptions();

	// enable command line option
	if (fullscreen !=2){
		option.fullScreenOnOpen = (fullscreen != 0);
	}
	if (autoclose !=2){
		option.closeOnEnd = (autoclose != 0);
	}
	if (loop !=2){
		option.endLessLoop = (loop != 0);
	}
	if (keepthisresolution !=2){
		option.keepResolution = (keepthisresolution != 0);
	}
	if (saveoptions !=2){
		option.saveOnExit = (saveoptions != 0);
	}

	if (volume!=-2){
	  if (volume >= 0 & volume <= 100){
	    volumeSlider->setValue(volume);
	    option.withAudio=1;
	  }
	  else {
	    option.withAudio=0;
          }
        }

	buttonLoop->setOn(option.endLessLoop);
	buttonSound->setOn(option.withAudio);
	
	show();
//***
// non GUI QT object
//***
	KDNDDropZone * dropZone = new KDNDDropZone( this, DndURL );
	connect( dropZone, SIGNAL( dropAction( KDNDDropZone *) ), SLOT( slotDropAction( KDNDDropZone *) ) );

	// install a timer : update frame every second
	openTimer = new QTimer( ); // create internal timer
	connect( openTimer, SIGNAL(timeout()), SLOT(slotOpenPopup()) );

	// install a timer : update info
	fpsTimer = new QTimer( ); // create internal timer
	connect( fpsTimer, SIGNAL(timeout()), SLOT(fps()) );

	// Generate the first event
	QTimer *timer = new QTimer();
	connect( timer, SIGNAL(timeout()), SLOT(slotPlay()) );
	timer->start(0, true);

	plWindow->addFile(file,false,true);
}

// Destructor
ZZplayer::~ZZplayer()
{
	DEBUGMSG("ZZplayer::~ZZplayer()");

	delete pluginmanager;
	delete fpsTimer;
	delete openTimer;
	DEBUGMSG("Bye Bye");
}

/** Define control panel
used only by the constructor */
inline void ZZplayer::createControl()
{
//***
// Definition de la vue principale
//***

	QBoxLayout *topLayout = new QVBoxLayout( this, extraSpace );

	topLayout->addSpacing(6);

//***
// Definition des boutons de controle
//***

	QBoxLayout *Layout = new QHBoxLayout();
	topLayout->addLayout(Layout);

	Layout->addSpacing(12);

	// Prev track button
	buttonPrev = new QPushButton(this);
	buttonPrev->setPixmap(prevPix);
	buttonPrev->setAccel('<');
	QToolTip::add(buttonPrev,i18n("Previous Track [<]"));
	buttonPrev->setFixedSize( 30, 20 );
	Layout->addWidget(buttonPrev);
	connect(buttonPrev,SIGNAL(clicked()), SLOT(slotPrev()));

	// Play button
	buttonPlay = new QPushButton(this);
	buttonPlay->setPixmap(playPix);
	buttonPlay->setAccel('P');
	QToolTip::add(buttonPlay,i18n("Play/Pause [P]"));
	buttonPlay->setFixedSize( 30, 20 );
	Layout->addWidget(buttonPlay);
	connect(buttonPlay,SIGNAL(clicked()), SLOT(slotPlay()));

	// Next track button
	buttonNext = new QPushButton(this);
	buttonNext->setPixmap(nextPix);
	buttonNext->setAccel('>');
	QToolTip::add(buttonNext,i18n("Next Track [>]"));
	buttonNext->setFixedSize( 30, 20 );
	Layout->addWidget(buttonNext);
	connect(buttonNext,SIGNAL(clicked()), SLOT(slotNext()));

	// Stop button
	buttonStop = new QPushButton(this);
	buttonStop->setPixmap(ICON(PIXMAP_PATH"/stop.xpm"));
	buttonStop->setAccel('S');
	QToolTip::add(buttonStop,i18n("Stop [S]"));
	connect(buttonStop,SIGNAL(clicked()), SLOT(slotStop()));
	buttonStop->setFixedSize( 30, 20 );
	Layout->addWidget(buttonStop);

	// Bouton Open
	buttonOpen = new QPushButton(this);
	buttonOpen->setPixmap(ICON(PIXMAP_PATH"/open.xpm"));
	buttonOpen->setAccel('O');
	QToolTip::add(buttonOpen,i18n("Open File [O]\nPush 1/2 second for more options"));
	buttonOpen->setFixedSize( 30, 20 );
	Layout->addWidget(buttonOpen);
	connect(buttonOpen,SIGNAL(pressed()), this,SLOT(slotPressOpenFile()));
	connect(buttonOpen,SIGNAL(released()), this,SLOT(slotReleaseOpenFile()));

	// menu recent
	openPopup = new QPopupMenu();
	openPopup->insertItem(i18n("Open &URL..."));
	openPopup->insertItem(i18n("Open &VCD..."));
	openPopup->insertSeparator();

	Layout->addSpacing(12);

	// Loop button
	buttonLoop = new QPushButton(this);
	buttonLoop->setPixmap(ICON(PIXMAP_PATH"/loop.xpm"));
	buttonLoop->setAccel('L');
	QToolTip::add(buttonLoop,i18n("Loop [L]"));
	buttonLoop->setFixedSize( 30, 22 );
	buttonLoop->setToggleButton(true);
	Layout->addWidget(buttonLoop);
	connect(buttonLoop,SIGNAL(clicked()),SLOT(slotLoop()));

	// Fullscreen button
	buttonFullscreen = new QPushButton(this);
	buttonFullscreen->setPixmap(ICON(PIXMAP_PATH"/fullscreen.xpm"));
	buttonFullscreen->setAccel('F');
	QToolTip::add(buttonFullscreen,i18n("Fullscreen Now [F]"));
	connect(buttonFullscreen,SIGNAL(clicked()), SLOT(slotFullScreenNow()));
	FIXE_TAILLE(buttonFullscreen);
	Layout->addWidget(buttonFullscreen);

	Layout->addSpacing(12);

	// Playlist button
	buttonPL = new QPushButton(this);
	buttonPL->setPixmap(ICON(PIXMAP_PATH"/list.xpm"));
	buttonPL->setAccel('I');
	QToolTip::add(buttonPL,i18n("Show/Hide PlayList [I]"));
	buttonPL->setFixedSize( 22, 22 );
	Layout->addWidget(buttonPL);
	connect(buttonPL,SIGNAL(clicked()),SLOT(slotShowPlaylist()));

	Layout->addStretch(1);

//***
// Position slider
//***
	topLayout->addSpacing(6);

	Layout = new QHBoxLayout(6);
	topLayout->addLayout(Layout);

	Layout->addSpacing(3);

	// Video Pixmap
	QPixmap pixvideo(ICON(PIXMAP_PATH"/video.xpm"));
	labelvideo = new QLabel(this);
	labelvideo->setPixmap( pixvideo );
	FIXE_TAILLE(labelvideo);
	Layout->addWidget(labelvideo,0); // ne peut pas grandir

	// slider
	sizeSlider = new QSlider(this);
	sizeSlider->setOrientation(QSlider::Horizontal);
	FIXE_HAUT(sizeSlider);
	sizeSlider->setSteps(1,1);
	sizeSlider->setRange(0,100);
	connect(sizeSlider,SIGNAL(sliderMoved(int)),this,SLOT(slotSizeSlider(int)) );
	Layout->addWidget(sizeSlider,1); // peut grandir

	Layout->addSpacing(6);

//***
// Volume, options, Help and FPS
//***
	topLayout->addSpacing(6);

	Layout = new QHBoxLayout(6);
	topLayout->addLayout(Layout);

	Layout->addSpacing(3);

	// Speaker button
	QIconSet speaker(soundPix,QIconSet::Small);
	speaker.setPixmap(nosoundPix,QIconSet::Small,QIconSet::Disabled);
	
	buttonSound = new QToolButton(this);
	buttonSound->setIconSet(speaker);
	buttonSound->setToggleButton(true);
	buttonSound->setOn(true);
	buttonSound->setAccel('M');
	QToolTip::add(buttonSound,i18n("Mute/Umute sound when available [P]"));
	connect(buttonSound,SIGNAL(clicked()), this,SLOT(slotWithAudio()));
	FIXE_TAILLE(buttonSound);
	Layout->addWidget(buttonSound);

	// Barre de volumeDialog
	volumeSlider = new QSlider(0,100,5,0, QSlider::Horizontal, this);
	volumeSlider->setTickmarks( QSlider::Above );
	volumeSlider->setTickInterval(10);
	volumeSlider->setTracking(true);
	connect(volumeSlider,SIGNAL(sliderMoved(int)),this,SLOT(slotVolumeSlider(int)) );
	FIXE_HAUT(volumeSlider);
	volumeSlider->setFixedWidth(100);
	Layout->addWidget(volumeSlider);

	// Options and Help Bonttons

	QBoxLayout *Layout2 = new QVBoxLayout(3);
	Layout->addLayout(Layout2);

	// Preferences Button
	buttonOption = new QPushButton(this);
	buttonOption->setPixmap(ICON(PIXMAP_PATH"/configure.xpm"));
	buttonOption->setAccel('C');
	QToolTip::add(buttonOption,i18n("Option dialog [C]"));
	FIXE_TAILLE(buttonOption);
	Layout2->addWidget(buttonOption);
	connect(buttonOption,SIGNAL(clicked()), SLOT(slotOptionDialog()));
	
	// Bouton help
	QPushButton *buttonHelp = new QPushButton(this);
	buttonHelp->setPixmap(ICON(PIXMAP_PATH"/help.xpm"));
	buttonHelp->setAccel('H');
	QToolTip::add(buttonHelp,i18n("Get &help"));
	FIXE_TAILLE(buttonHelp);
	Layout2->addWidget(buttonHelp);
	connect(buttonHelp,SIGNAL(clicked()), SLOT(slotHelpContents()));

	Layout2->addSpacing(3);

	Layout->addStretch();

	// Info lecture
	labelfps = new QLabel( i18n(labelTemplate), this, "" );
	labelfps->setAlignment( AlignTop );
	labelfps->setFrameStyle( QFrame::Box | QFrame::Sunken );
	FIXE_TAILLE(labelfps);
	Layout->addWidget(labelfps);
	
	topLayout->freeze(0,0);
}

// Read from config file
inline void ZZplayer::readOptions()
{
 	DEBUGMSG("ZZplayer::readOptions(...)");
	// reload last config
	KConfig* config = kapp->getConfig();
	if (config == NULL)
		return;
	// position and geometry of the window
	QPoint pDefaultPos( 100, 100 );
	config->setGroup( "General Options" );
	QPoint position = config->readPointEntry( "Positon", &pDefaultPos );
	setGeometry(position.x(),position.y(),0,0);
	// CheckBox saveOptionsOnExit
	option.saveOnExit=config->readBoolEntry("saveOptionsOnExit" ,true);
	// CheckBox videoLoop
	option.endLessLoop=config->readBoolEntry("videoLoop" ,false);
	// CheckBox Audio
	option.withAudio=config->readBoolEntry("withAudio" ,true);
	// CheckBox fullScreen
	option.fullScreenOnOpen=config->readBoolEntry("fullScreen" ,false);
	// autoClose CheckBox
	option.closeOnEnd=config->readBoolEntry("autoClose" ,false);
	// keepThisResolution CheckBox
	option.keepResolution=config->readBoolEntry("keepThisResolution" ,false);
	// keepratio CheckBox
	option.keepRatio=config->readBoolEntry("keepRatio" ,false);
	//  multipleInstance CheckBox
	option.multipleInstance=config->readBoolEntry("multipleInstance" ,false);
	// bilinearFilter CheckBox
	option.bilinearFilter=config->readBoolEntry("bilinearFilter" ,false);
	// Volume
	// set the volume slide
	volumeSlider->setValue(config->readNumEntry("audioVolume",VOL_DEF));
	// set popupOnClick
	option.popupOnClick=config->readBoolEntry("popupOnClick",false);

	// recent files
	int z;
	char name[16];
	QString s;

	for (z = 0; z < 5; z++) {
		sprintf(name, "Recent%d", z + 1);
		s = config->readEntry(name);
		if (!s.isEmpty())
			openPopup->insertItem(s);
	}
}


// Save to config file
void ZZplayer::slotSaveOptions()
{
	int z;
	char name[16];

 	DEBUGMSG("ZZplayer::slotSaveOptions(...)");
	KConfig* config = kapp->getConfig();
	if (config == NULL)
		return;
	config->setGroup( "General Options" );
	config->writeEntry( "Positon", pos() );
	config->writeEntry("saveOptionsOnExit" ,option.saveOnExit);
	config->writeEntry("fullScreen" ,option.fullScreenOnOpen);
	config->writeEntry("keepThisResolution" ,option.keepResolution);
	config->writeEntry("keepRatio" ,option.keepRatio);
	config->writeEntry("autoClose" ,option.closeOnEnd);
	config->writeEntry("videoLoop" ,option.endLessLoop);
	config->writeEntry("multipleInstance" ,option.multipleInstance);
	config->writeEntry("bilinearFilter" ,option.bilinearFilter);
	config->writeEntry("withAudio" ,option.withAudio);
	config->writeEntry("popupOnClick" ,option.popupOnClick);
	config->writeEntry("audioVolume" ,volumeSlider->value());
	// fichiers recents
	for (z = IDA; z < (int) openPopup->count(); z++)
		{
		sprintf(name, "Recent%d", z - (IDA-1));
		config->writeEntry(name, openPopup->text(z));
		}

	config->sync();
}


void ZZplayer::slotDropAction( KDNDDropZone * _dropZone )
{
 	DEBUGMSG("ZZplayer::slotDropAction(...)");
 // get URL list dropped
	QStrList urls = _dropZone->getURLList();
	plWindow->addList(urls);
}

/****
** File management
****/

void ZZplayer::loadFile()
{
// loadFile does only load and start playing a given file
// So loadFile don't have to manage GUI.
// But this not the case for now, that will change soon!
 	DEBUGMSG("ZZplayer::loadFile(...)");

// Close any loaded file
// In the future loadFile will only loading the current file
// and a playFile method will play it!
	closeFile();

	QString *pfile=plWindow->currentFile();
	if (pfile == NULL)
		{
		stateStop();
		return;
		}
	// set the appropriate volume
	int volume=option.withAudio?volumeSlider->value():0;

	genmpeg=pluginmanager->openStream(*pfile,option.withAudio,1,option.windowId,kdeCallback);
	if (genmpeg == NULL)
	  {
	  KMsgBox::message( this, *pfile,i18n("Cannot create stream."));
	  return;
	  }
	if (genmpeg->error() != NULL)
	  {
	  KMsgBox::message( this, *pfile,genmpeg->error());
	  delete genmpeg;
	  genmpeg=NULL;
	  return;
	  }

    // set the parameters
	genmpeg->setVolume(volume);
	genmpeg->setBilinearFilter(option.bilinearFilter);
	genmpeg->setKeepThisResolution(option.keepResolution);
	genmpeg->setKeepRatio(option.keepRatio);
	fscreen=option.fullScreenOnOpen;
	genmpeg->fullScreen(fscreen);

	// Change GUI state
	statePlay();

	// On place le fichier dans les fichiers recents
	int z = (int) openPopup->count();
	while (z > IDA)
		{
		z--;
		if (*pfile == openPopup->text(z))
			openPopup->removeItemAt(z);
		}
	openPopup->insertItem(*pfile, 0, IDA);
	if (openPopup->count() > 5+IDA)
		openPopup->removeItemAt(5+IDA);
	for (z = 0; z < 5+IDA; z++)
		openPopup->setId(z, z);

	// print some info about the file
	StreamInfo info;
	genmpeg->getFullInfo(&info);

  if ( info.has_audio && info.has_video ) {
    DEBUGMSG( *pfile << " : MPEG system stream (audio/video)");
  } else if ( info.has_audio ) {
    DEBUGMSG( *pfile << " : MPEG audio stream");
  } else if ( info.has_video ) {
    DEBUGMSG( *pfile << " : MPEG video stream");
  }
  if ( info.has_video ) {
    DEBUGMSG("    Video " <<info.width << "x" <<info.height << " resolution");
  }
  if ( info.has_audio ) {
    DEBUGMSG("    Audio : " << info.audio_string);
  }
  if ( info.total_size ) {
    DEBUGMSG("    Size : " << info.total_size);
  }
  DEBUGMSG("End loadFile...");
}

void ZZplayer::increaseSound(int val)
{
if (genmpeg->hasSound() && option.withAudio)
  {
  volumeSlider->setValue(volumeSlider->value()+val);
  genmpeg->setVolume(volumeSlider->value());
  }
}

void ZZplayer::decreaseSound(int val)
{
if (genmpeg->hasSound() && option.withAudio)
  {
  volumeSlider->setValue(volumeSlider->value()-val);
  genmpeg->setVolume(volumeSlider->value());
  }
}

void ZZplayer::fps()
{

 	DEBUGMSG("ZZplayer::fps(...)");
//Update seek slider, frame and  Fps
	if ((genmpeg == NULL) || (state != Play))
		return;

	QString labeltext;
	int seek;
	int frame;
	double fps;

 	genmpeg->getInfo(&seek, &fps, &frame);

 	labeltext.sprintf(" Fps : %.2f\n Frame : %.5i ",fps,frame);
 	labelfps->setText(labeltext);
 	sizeSlider->setValue(seek);

 	// if we are on the end of file
 	if (genmpeg->status() == STREAM_STOPPED)
 		{
		DEBUGMSG("The file was finished");
		if (plWindow->nextFile() != NULL)
			{
			loadFile();
			return;
			}
 		if (option.endLessLoop)
			{
			plWindow->firstFile();
 			loadFile();
			return;
			}
 		if (option.closeOnEnd)
			{
 			close();
			return;
			}
		closeFile();
		stateStop();
 		}
}


// Ferme le fichier courant
void ZZplayer::closeFile()
{
// CloseFile does only closing the current file
// (ie freeing associate ressources)
// So closeFile don't have to manage GUI.
  DEBUGMSG("ZZplayer::closeFile(...)");
  if (genmpeg == NULL)
	{
    DEBUGMSG("Warning : try to close NULL file.");
	}
  else
	{
    pluginmanager->closeStream(genmpeg);
    genmpeg=NULL;
	}
}


/****
** Control window management
****/

// Button "Previous track" was clicked
void ZZplayer::slotPrev()
{
 	DEBUGMSG("ZZplayer::slotPrev(...)");

	QString *ret=plWindow->prevFile();
	if ((ret == NULL) && (option.endLessLoop))
		plWindow->lastFile();
	else if ((state == Stop) || (ret == NULL))
		return;
	loadFile();
}

// Button play was clicked
void ZZplayer::slotPlay()
{
 	DEBUGMSG("ZZplayer::slotPlay(...)");
	// if previous state was stop
	if ((genmpeg == NULL) || (state == Stop))
		{
		loadFile();
		return;
		}
	// we are playing a file, so we manage a pause button
	genmpeg->pause();
	if (state == Play)
		{
		buttonPlay->setDown(true);
		state=Pause;
		}
	else
		{
		buttonPlay->setDown(false);
		state=Play;
		}
}

// Button "Next track" was clicked
void ZZplayer::slotNext()
{
 	DEBUGMSG("ZZplayer::slotNext(...)");

	QString *ret=plWindow->nextFile();
	if ((ret == NULL) && (option.endLessLoop))
		plWindow->firstFile();
	else if ((state == Stop) || (ret == NULL))
		return;
	loadFile();
}


void ZZplayer::slotStop()
{
 	DEBUGMSG("ZZplayer::stop(...)");
//
	closeFile();
	stateStop();
}

// Button "Next track" was clicked
void ZZplayer::slotFileChange()
{
 	DEBUGMSG("ZZplayer::slotFileChange(...)");

	closeFile();
	if (state == Stop)
		return;
	loadFile();
}


void ZZplayer::slotPressOpenFile()
{
 	DEBUGMSG("ZZplayer::slotPressOpenFile(...)");
	openTimer->start( 500, true ); // Start timer, on timeout, the mouse event wasn't a click!
}

void ZZplayer::slotOpenPopup()
{
 	DEBUGMSG("ZZplayer::slotOpenPopup(...)");
	int ret=openPopup->exec(buttonOpen->mapToGlobal(QPoint(0,buttonOpen->height())));
	buttonOpen->setDown(false);
	if (ret < 0)
		return;
	selectPopup(ret);
}

void ZZplayer::slotReleaseOpenFile()
{
 	DEBUGMSG("ZZplayer::slotReleaseOpenFile(...)");
	QString file;
	openTimer->stop(); // Stop timer (even it was disabled)

	if (option.popupOnClick)
		{
		slotOpenPopup();
		return;
		}
	file=KFileDialog::getOpenFileName(file,	"*.mpg *.mpeg *.mpe *.MPG *.MPEG *.MPE *.MP3 *.mp3|Mpeg Files\n"
							"*|All Files (*)\n");
	if (!file.isEmpty())
		{
		plWindow->addFile(file);
		}
}


// ouvre un fichier de la liste
void ZZplayer::selectPopup(int id)
{
// That not a good way!!
// Must be corrected!!
// As URL mean Universal, local file must be a particuliar case, not the inverse!
	QString url;

 	DEBUGMSG("ZZplayer::selectPopup(...)");
	if (id == 2)
		return;
	if (id == 0)
		{
		url=KFileDialog::getOpenFileURL(url, "*.mpg *.mpeg *.mpe *.MPG *.MPEG *.MPE *.MP3 *.mp3|Mpeg Files\n"
							"*|All Files (*)\n");
		if (url.isEmpty())
			return;
		}
	else if (id == 1)
		{
		url="vcd:"+option.VCDDevice;

		}
	else
		{
		url=openPopup->text(id);
		}

	closeFile();
	plWindow->addFile(url);
	loadFile();
}

void ZZplayer::slotLoop()
{
 	DEBUGMSG("ZZplayer::setLoop(...)");
	option.endLessLoop=buttonLoop->isOn();
}


// Slider for volume
void ZZplayer::slotVolumeSlider(int vol)
{
 	DEBUGMSG("ZZplayer::slotVolumeSlider(...)");
	if(genmpeg == NULL)
		return;
	genmpeg->setVolume(vol);
}

// Slider for the search bar
void ZZplayer::slotSizeSlider(int seek)
{
 	DEBUGMSG("ZZplayer::slotSizeSlider(...)");
	if (genmpeg == NULL) // Project : Can start a movie when you want!
		return;
	genmpeg->setSeek(seek);
}

// Switch to FullScreen Now
void ZZplayer::slotFullScreenNow()
{
 	DEBUGMSG("ZZplayer::slotFullScreenNow(...)");
	if (genmpeg != NULL){
		genmpeg->fullScreen(1);
	}
}

// Switch to keep Ratio
void ZZplayer::slotKeepRatio()
{
 	DEBUGMSG("ZZplayer::slotKeepRatio(...)");
	if (genmpeg != NULL){
		genmpeg->setKeepRatio(option.keepRatio);
	}
}


// CheckedBox pour avoir l'image sans le son
void ZZplayer::slotWithAudio()
{
 	DEBUGMSG("ZZplayer::slotWithAudio(...)");
	option.withAudio = !option.withAudio;
	volumeSlider->setEnabled(option.withAudio);
	if ((genmpeg != NULL) && genmpeg->hasSound()) // Well, if this event occur that mean
		{                                         // that genmpeg->hasSound() is true, but...
  		genmpeg->setVolume(option.withAudio?volumeSlider->value():0);
		}
	buttonSound->setOn(option.withAudio);
}

void ZZplayer::slotOptionDialog()
{
 	DEBUGMSG("ZZplayer::slotOptionDialog(...)");
	if (optionBox == NULL)
		{
		optionBox= new dialogBox(&option);
		connect(optionBox,SIGNAL(applyOptions()),SLOT(slotKeepRatio()));
		connect(optionBox,SIGNAL(saveOpt()),SLOT(slotSaveOptions()));
		}
	optionBox->show();
}

void ZZplayer::slotShowPlaylist()
{
 	DEBUGMSG("ZZplayer::slotShowPlaylist(...)");
	if (plWindow->isVisible())
		plWindow->hide();
  	else
		plWindow->show();
}


void ZZplayer::slotHelpContents()
{
 	DEBUGMSG("ZZplayer::slotHelpContents(...)");
	kapp->invokeHTMLHelp("ZZplayer/zzplayer.html", "");
}

/****
** Change state of control window
****/

void ZZplayer::statePlay()
{
 	DEBUGMSG("ZZplayer::stateplay(...)");
	// Update internel state
	state=Play;
	fpsTimer->start( 500 );
  // enable the video slide
	sizeSlider->setEnabled(true);
	labelvideo->setEnabled(true);
	// update sound state
	bool tmp=genmpeg->hasSound();
	volumeSlider->setEnabled(option.withAudio && tmp);
	buttonSound->setOn(option.withAudio);
	buttonSound->setEnabled(tmp);
	// switch play button in pause button!
	buttonPlay->setPixmap(pausePix);
	buttonPlay->setToggleButton(true);
	buttonPlay->setDown(false);
 	DEBUGMSG("exit ZZplayer::stateplay(...)");
}

void ZZplayer::stateStop()
{
 	DEBUGMSG("ZZplayer::stateStop(...)");

	// Update internel state
	fpsTimer->stop();
	state=Stop;
  // enable the video slide
	sizeSlider->setEnabled(false);
	labelvideo->setEnabled(false);
	// switch pause button in play button!
	buttonPlay->setDown(false);
	buttonPlay->setToggleButton(false);
	buttonPlay->setPixmap(playPix);
	// reset file position to zero.
	sizeSlider->setValue(0);
	// restore sound button in default state	
	volumeSlider->setEnabled(true);
	buttonSound->setEnabled(true);
	buttonSound->setOn(true);
}

/****
** reomplement closeEnvent
****/

void ZZplayer::closeEvent ( QCloseEvent * e )
{
	DEBUGMSG("ZZplayer::closeEvent()");

	// stop all existing timer
	fpsTimer->stop();
	openTimer->stop();
	// Save option on exit
	if (option.saveOnExit)
		slotSaveOptions();
	// Close any file in case of...
	closeFile();
	if (optionBox != NULL)
		optionBox->close();
	e->accept();
}
