//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//  Home    : http://www.chez.com/tsc/zzplayer/zzplayer.html
//
//	Requires the Qt widget libraries, available at no cost at
//      http://www.troll.no
//
//  File    : dialogBox.cpp
//  Author  : Frederic Grasset
//  E-Mail  : frederic.grasset@wanadoo.fr

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#include "dialogBox.h"
#include "dialogBox.moc"

#include "zzoptions.h"

dialogBox::dialogBox(ZZoptions *opt)
:QTabDialog()
{
	if (opt == NULL)
		return;
	p=opt;

	setCaption(i18n("ZZplayer : Options"));

// We add the first tab
	QWidget *tab = first_tab(this);
	addTab(tab,i18n("&Preferences"));

// We add the second tab
	tab = second_tab(this);
	addTab(tab,i18n("Appea&rance"));

// We add the third tab
	tab = third_tab(this);
	addTab(tab,i18n("&About ZZplayer"));

// We setup dialog's buttons ("ok", "apply", "cancel")
	setCancelButton();
	setApplyButton();
	setOkButton();
	
	resize(0,0);
	
	connect(this,SIGNAL(applyButtonPressed()),SLOT(applyAction()));
}

dialogBox::~dialogBox()
{
}


// Option that was in this dialogBox does not affect the currently
// playing file!!

void dialogBox::saveNow()
{
	applyAction();
	emit saveOpt();
}


void dialogBox::applyAction()
{
	p->fullScreenOnOpen=fullScreenOnOpen->isChecked();
	p->keepResolution=keepResolution->isChecked();
	p->keepRatio=keepRatio->isChecked();
	p->closeOnEnd=closeOnEnd->isChecked();
	p->saveOnExit=saveOnExit->isChecked();
	p->popupOnClick=popupOnClick->isChecked();
	p->multipleInstance=multipleInstance->isChecked();
	p->bilinearFilter=bilinearFilter->isChecked();
	p->VCDDevice=Device->text();
	emit applyOptions();
}

// Create the first tab
// for use by the constructor only
QWidget * dialogBox::first_tab(QWidget *parent)
{
	QWidget *tab = new QWidget(parent);

// We organize our widgets by line
	QBoxLayout *topLayout = new QVBoxLayout( tab, 6 );
	
// 1st widget's line : "stat-up" options
// A groupbox with a grid layout management
	QGroupBox *box = new QGroupBox(i18n("Start-up options"),tab);
	topLayout->addWidget(box);

	QGridLayout *grid = new QGridLayout( box,1, 3,20 );
	grid->setColStretch(1,10);

	multipleInstance=new QCheckBox(i18n("Allow Multiple Instance"),box);
	multipleInstance->setChecked(p->multipleInstance);
	FIXE_TAILLE(multipleInstance);
	grid->addWidget(multipleInstance,0,0);

	fullScreenOnOpen=new QCheckBox(i18n("FullScreen When open"),box);
	fullScreenOnOpen->setChecked(p->fullScreenOnOpen);
	FIXE_TAILLE(fullScreenOnOpen);
	grid->addWidget(fullScreenOnOpen,0,2);

	grid->activate();

// 2nd widget's line : "shutdown" options
// A groupbox with a grid layout management
	box = new QGroupBox(i18n("Ending options"),tab); // yes, we loose previous widget ref!
	topLayout->addWidget(box);

	grid = new QGridLayout( box,1, 3,20 ); // yes, we re-use this name too!
	grid->setColStretch(1,10);

	closeOnEnd=new QCheckBox(i18n("Close At End of File"),box);
	closeOnEnd->setChecked(p->closeOnEnd);
	FIXE_TAILLE(closeOnEnd);
	grid->addWidget(closeOnEnd,0,0);

	saveOnExit=new QCheckBox(i18n("Save Options On Exit"),box);
	saveOnExit->setChecked(p->saveOnExit);
	FIXE_TAILLE(saveOnExit);
	grid->addWidget(saveOnExit,0,2);

	grid->activate();

// 4th widget's line : VCD device name
// a Line edit with a naming label
	QBoxLayout *Layout = new QHBoxLayout(6,0);
	topLayout->addLayout(Layout);

	QLabel *ld=new QLabel(i18n("VCD device : "),tab);
	FIXE_TAILLE(ld);
	Layout->addWidget(ld);

	Device = new QLineEdit(tab);
	FIXE_HAUT(Device);
	Device->setText(p->VCDDevice);
	Layout->addWidget(Device);

//
	topLayout->addStretch(10);


// 6th widget's line : "Save now" button
// Normally, this button doesn't stretch and will be rigth aligned
	QPushButton *save = new QPushButton(i18n("Save Now"),tab);
	FIXE_TAILLE(save);
	topLayout->addWidget(save,0,AlignRight);
	connect(save,SIGNAL(clicked()),SLOT(saveNow()));

// We finish this tab, we can activate layout management
	topLayout->activate();

	return tab;
}
// Create the second tab
// for use by the constructor only
QWidget * dialogBox::second_tab(QWidget *parent)
{
	QWidget *tab = new QWidget(parent);

// We organize our widgets by line
	QBoxLayout *topLayout = new QVBoxLayout( tab, 6 );

// 1st widget's line : "stat-up" options
// A groupbox with a grid layout management
	QGroupBox *box = new QGroupBox(i18n("Video"),tab);
	topLayout->addWidget(box);

	QGridLayout *grid = new QGridLayout( box,2, 3,20 );
	grid->setColStretch(1,10);

	bilinearFilter=new QCheckBox(i18n("Bilinear Filter (software)"),box);
	bilinearFilter->setChecked(p->bilinearFilter);
	FIXE_TAILLE(bilinearFilter);
	grid->addWidget(bilinearFilter,0,0);

	keepResolution=new QCheckBox(i18n("Don't do \"Mode Switch\" (when fullscreen)"),box);
	keepResolution->setChecked(p->keepResolution);
	FIXE_TAILLE(keepResolution);
	grid->addWidget(keepResolution,0,2);

	keepRatio=new QCheckBox(i18n("Keep video ratio"),box);
	keepRatio->setChecked(p->keepRatio);
	FIXE_TAILLE(keepRatio);
	grid->addWidget(keepRatio,1,0);


	grid->activate();

// 2nd widget's line : "shutdown" options
// A groupbox with a grid layout management
	box = new QGroupBox(i18n("Control"),tab); // yes, we loose previous widget ref!
	topLayout->addWidget(box);

	grid = new QGridLayout( box,1, 3,20 ); // yes, we re-use this name too!
	grid->setColStretch(1,10);

	popupOnClick=new QCheckBox(i18n("Popup file menu when click on \"Open\""),box);
	popupOnClick->setChecked(p->popupOnClick);
	FIXE_TAILLE(popupOnClick);
	grid->addWidget(popupOnClick,0,0);

    grid->activate();

// 3th widget's line : "playlist" options
// A groupbox with a grid layout management
//	box = new QGroupBox(i18n("Playlist"),tab); // yes, we loose previous widget ref!
//	topLayout->addWidget(box);

//	grid = new QGridLayout( box,1, 3,20 ); // yes, we re-use this name too!
//	grid->setColStretch(1,10);

//    grid->activate();
//
	topLayout->addStretch(10);

// last widget's line : "Save now" button
// Normally, this button doesn't stretch and will be rigth aligned
	QPushButton *save = new QPushButton(i18n("Save Now"),tab);
	FIXE_TAILLE(save);
	topLayout->addWidget(save,0,AlignRight);
	connect(save,SIGNAL(clicked()),SLOT(saveNow()));

// We finish this tab, we can activate layout management
	topLayout->activate();

	return tab;
}

/** Create the third tab
for use in the constructor only */
QWidget * dialogBox::third_tab ( QWidget *parent )
{
	QLabel *tab = new QLabel(i18n(INFO_MESSAGE),parent);
	tab->setAlignment(AlignCenter);
	tab->setMargin(6);
	tab->setMinimumSize(tab->sizeHint());

	return tab;
}
