//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//  Home  : http://zzplayer.sourceforge.net
//
//	Requires the Qt widget libraries, available at no cost at
//      http://www.troll.no
//	
//  File  : zzplayer.h	
//  Modif :
//    - Sept 2000, Frederic Grasset
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#ifndef ZZPLAYER_H
#define ZZPLAYER_H

#include <stdlib.h>
#include <string.h>


#include <kapp.h>
#include <kiconloader.h>
#include <kfiledialog.h>
#include <kmsgbox.h>
#include <kstdaccel.h>
#include <kfm.h>
#include <kurl.h>

#include <qdialog.h>
#include <qlayout.h>
#include <drag.h>
#include <qpopupmenu.h>
#include <qkeycode.h>
#include <qlabel.h>
#include <qslider.h>
#include <qtimer.h>
#include <qpixmap.h>
#include <qfileinfo.h>

#include <qtooltip.h>
#include <qpushbutton.h>
#include <qtoolbutton.h>

#include "zzoptions.h"
#include "playlist.h"
#include "dialogBox.h"
#include "../../plugins/Plugin.h"
#include "../../plugins/PluginManager.h"


#define INFO_MESSAGE "ZZplayer "##VERSION"\n\n"\
"Copyright (c) 1999 Nicolas Vignal <nicolas.vignal@fnac.net>\n\n"\
"This program is free software; you can redistribute it\nand/or modify"\
" it under the terms of the GNU General Public\nLicense as"\
" published by"\
" the Free Software Foundation; either\nversion 2 "\
"of the License, or (at your option) any later version.\n\n"\
"The program is distributed in hopes that it will be useful,\n"\
"but WITHOUT ANY WARRANTY; without even the implied\nwarranty of"\
" MERCHANTABILITY or FITNESS FOR A\n"\
"PARTICULAR PURPOSE. See the GNU General Public\n"\
"License for more details.\n\n"\
"You should have received a copy of the GNU General\nPublic License along"\
" with this program.  If not, write\nto the Free Software Foundation, Inc.,"\
" 675 Mass Ave,\n"\
"Cambridge, MA 02139, USA."

enum SMPEGState {Stop, Play, Pause};

class ZZplayer : public QWidget
{
	Q_OBJECT
//Private
	QPixmap	prevPix;
	QPixmap	playPix;
	QPixmap	nextPix;
	QPixmap	pausePix;
	QPixmap	soundPix;
	QPixmap	nosoundPix;

	QLabel	*labelfps;

	QPushButton   *buttonStop;
	QPushButton   *buttonPrev;
	QPushButton   *buttonPlay;
	QPushButton   *buttonNext;
	QPushButton   *buttonOpen;
	QPushButton   *buttonLoop;
	QPushButton   *buttonPL;
	QPushButton   *buttonFullscreen;
	QPushButton   *buttonOption;

	QToolButton    *buttonSound;

	QSlider *volumeSlider;
	QLabel *labelvideo;
	QSlider *sizeSlider;

	QPopupMenu *openPopup;

	QTimer *openTimer;
	QTimer *fpsTimer;

	ZZplaylist *plWindow;
	dialogBox *optionBox;

	// Data
	SMPEGState state;
//	QString fileName;
	ZZoptions option;
	PluginManager *pluginmanager;

//public
public:
	ZZplayer(QString file, int fullscreen, int volume, int autoclose , int loop, int keepthisresolution, int saveoptions, int windowID);
	~ZZplayer();

//	void loadURL( QString & );
	void loadFile();

	void increaseSound(int val);
	void decreaseSound(int val);

// Public data
	Plugin  *genmpeg;
	int fscreen;

// slots
public slots:
	void slotSaveOptions();
	void slotDropAction( KDNDDropZone * );
	void slotPrev();
	void slotPlay();
	void slotNext();
	void slotStop();
	void slotFileChange();
	void slotPressOpenFile();
	void slotOpenPopup();
	void slotReleaseOpenFile();
	void slotLoop();
	void slotFullScreenNow();
	void slotKeepRatio();
	void slotOptionDialog();
	void slotShowPlaylist();
	void slotHelpContents();
	void slotVolumeSlider(int vol);
	void slotSizeSlider(int fileoffset);
	void slotWithAudio();

	void fps();

private: // Private methods
  /** Define control panel
used only by the constructor */
	void createControl();
	void readOptions();
	void closeFile();
	void selectPopup(int id);

	void statePlay();
	void stateStop();

	virtual void closeEvent ( QCloseEvent * e );
};

// used in callback function, initialized in kdemain.cpp
static int kdeCallback( ZZ_EventStruct *event);
extern ZZplayer *player;



#endif
