//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//
//	Requires the Qt widget libraries, available at no cost at
//      http://www.troll.no
//	
//  Author  : Nicolas Vignal
//  E-Mail  : nicolas.vignal@fnac.net
//  Home    : http://zzplayer.sourceforge.net

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#include <stdio.h>
#include <unistd.h>
#include "../../plugins/Plugin.h"
#include "../../plugins/PluginManager.h"

#ifndef NOGUI_CPP
#define NOGUI_CPP

int done;
Plugin  *genmpeg=NULL;
int fscreen=0;
int statepause=0;
int audio=60;

int callback( ZZ_EventStruct *event)
{
  if (event->type==ZZ_MOUSEBUTTONDOWN) {
  	if ((char)event->data==MOUSEBUTTON_LEFT){
  		fscreen=!fscreen;
  		genmpeg->fullScreen(fscreen);
  	}
  	else if ((char)event->data==MOUSEBUTTON_RIGHT){
		  genmpeg->pause();
		  statepause=!statepause;
  	}
  	else if ((char)event->data==MOUSEBUTTON_MIDDLE){
  	  // What we can do if the middle button is pressed ? Any idea ?
  	}
  	else if ((char)event->data==MOUSEBUTTON_SCROLLUP){
		  if ( audio < 100 && audio != -1) {
    	  audio=audio+5;
	      genmpeg->setVolume(audio);
  	  }
  	}
  	else if ((char)event->data==MOUSEBUTTON_SCROLLDOWN){
		  if ( audio > 0 ) {
    	  audio=audio-5;
	      genmpeg->setVolume(audio);
  	  }
  	}
  }
  else if (event->type==ZZ_QUIT){
    done=1;
  }
	else {
	  DEBUGMSG("event : " << event->type)
	}
	
	return 0;
}

int nogui(int argc, char *argv[],int fullscreen, int volume, int autoclose , int loop, int keepthisresolution, int saveoptions, int windowID)
{
  char *file=MPEG_PATH"/ZZplayer.mpg";;
	PluginManager  *pluginmanager=NULL;
	int withAudio=1;

 	pluginmanager = new PluginManager();

 	done = 0;

 	// check if we want sound
 	if (volume==-1){
 	  withAudio=0;
  	audio=-1;
  }

 	// check if we give some options about fullscreen or loop mode
 	if (fullscreen==2){
 	  fullscreen=0;
 	}
 	if (loop==2){
 	  loop=0;
 	}

 	// check if we gave a file name
	if ( !((argc == 1) || (argc == optind))){
		file=argv[optind];
	}

  do
 	{
    genmpeg=pluginmanager->openStream(file,withAudio,1,windowID,callback);

  	if (genmpeg->error()){
  	  fprintf(stderr, "Cannot play the file %s\nError returned : %s\n",argv[1],genmpeg->error());
    	delete genmpeg;
  	  genmpeg=NULL;
  		return 1;
 		}

	  // check if we give a volume. If not,we use the default volume 60
	  if ((volume!=-2) && withAudio){
	    audio=volume;
	  }
	  if (withAudio) {
	    genmpeg->setVolume(audio);
	  }

	  genmpeg->setKeepThisResolution(keepthisresolution);

	  if (fullscreen){
		  fscreen=1;
		  genmpeg->fullScreen(fscreen);
    }

    while (!done && (genmpeg->status()!=STREAM_STOPPED || statepause)){
      usleep(500000);  // wait 250ms
    }
    pluginmanager->closeStream(genmpeg);
  }
  while ( loop && !done);

  delete pluginmanager;
  return 0;
}

#endif