//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//
//	Requires the Qt widget libraries, available at no cost at
//      http://www.troll.no
//	
//  Author  : Nicolas Vignal
//  E-Mail  : nicolas.vignal@fnac.net
//  Home    : http://zzplayer.sourceforge.net

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#include <getopt.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>


enum {NOGUI,KDEGUI,GNOMEGUI,EQDEGUI};


#ifdef HAVE_CONFIG_H

#include "config.h"

#endif

#ifdef HAVE_KDE_GUI
int kdegui(int argc, char *argv[],int fullscreen, int volume, int autoclose , int loop, int keepthisresolution, int saveoptions, int windowID);
#endif

#ifdef HAVE_GNOME_GUI
extern "C" {
int gnomegui(int argc, char *argv[],int fullscreen, int volume, int autoclose , int loop, int keepthisresolution, int saveoptions, int windowID);
}
#endif

#ifdef HAVE_EQDE_GUI
int eqdegui(int argc, char *argv[],int fullscreen, int volume, int autoclose , int loop, int keepthisresolution, int saveoptions, int windowID);
#endif



struct option long_opt[]={
{"fullscreen",optional_argument,NULL,'f'},
{"autoclose",optional_argument,NULL,'c'},
{"audio",optional_argument,NULL,'a'},
{"loop",optional_argument,NULL,'l'},
{"keepresolution",optional_argument,NULL,'r'},
{"nogui",no_argument,NULL,'n'},
{"gnomegui",no_argument,NULL,'g'},
{"kdegui",no_argument,NULL,'k'},
{"eqdegui",no_argument,NULL,'e'},
{"windowID",required_argument,NULL,'W'},
{"saveoptions",optional_argument,NULL,'s'},
{"help",no_argument,NULL,'h'},
{"version",no_argument,NULL,'V'},
{NULL,0,NULL,0}
};

char *optstring="fwcqa::lortngkeW:sehV";

void printversion ()
{
   printf("ZZplayer version "VERSION "\n");
}

void printusage ()
{
    printf(
"Usage: zzplayer [options] file ...\n"
"Where the options are one of:\n"
"	--fullscreen[=yes], -f      Play MPEG in fullscreen mode\n"
"	--fullscreen=no, -w         Play MPEG in window mode\n"
"	--autoclose[=yes], -c       Play MPEG and close the player\n"
"	--autoclose=no, -q          Play MPEG and don't close the player\n"
"	--audio[=volume], -a        Set audio volume in range [0..100] (default = 60 and -1 disable audio)\n"
"	--loop[=yes], -l            Play MPEG over and over\n"
"	--loop=no, -o               Play MPEG one time\n"
"	--keepresolution[=yes], -r  Force the fullscreen resolution to the current resolution\n"
"	--keepresolution=no, -t     Allow the fullscreen mode to chose the best resolution\n"
"	--windowID, -W              Specify a window Id to draw into\n"
"	--saveoptions[=yes], -s     Save this command line options\n"
"	--saveoptions=no, -e        Don't save options\n"
"	--help, -h                  This menu\n"
"	--version, -V               print version and quit\n\n"
"GUI options :\n"
#ifdef HAVE_KDE_GUI
"	--kdegui, -k                launch ZZplayer with the KDE GUI\n"
#endif
#ifdef HAVE_GNOME_GUI
"	--gnomegui, -g              launch ZZplayer with the Gnome GUI\n"
#endif
#ifdef HAVE_EQDE_GUI
"	--eqdegui, -e               launch ZZplayer with the Equinox GUI\n"
#endif
"	--nogui, -n                 Launch ZZplayer without GUI (default)\n\n"
"VCD support :\n"
"To read a VCD use vcd:/dev/cdrom as file.\n"
"You can specify a track like this vcd:/dev/cdrom:1\n"
"The default track is 2.\n\n");
}

int val_opt(char *opt)
{
	if (opt==NULL)
		return 1;
	if (strcmp(opt,"no") == 0)
		return 0;
	if (strcmp(opt,"yes") == 0)
		return 1;
	return 2;
}

int main(int argc, char* argv[])
{
	int option_index=0;
	int ret,v;
        int gui=EQDEGUI;   // will be NOGUI
	int volume=-2;
	int fullscreen=2;
	int autoclose=2;
	int loop=2;
	int keepresolution=2;
	int windowID=0;
	int saveoptions=2;
	bool showplayer=true;


	while (true)
		{
		ret=getopt_long(argc, argv, optstring, long_opt, &option_index);
		if (ret < 0)
			break;
		switch(ret)
			{
			case 'a':
				if (optarg == NULL)
					break;
				v=atoi(optarg);
				if ((v < -1) || (v > 100))
					break;
				volume=v;
				break;
			case 'w':
				fullscreen=0;
				break;
			case 'f':
				fullscreen=val_opt(optarg);
				break;
			case 'q':
				autoclose=0;
				break;
			case 'c':
				autoclose=val_opt(optarg);
				break;
			case 'o':
				loop=0;
				break;
			case 'l':
				loop=val_opt(optarg);
				break;
			case 't':
				keepresolution=0;
				break;
			case 'r':
				keepresolution=val_opt(optarg);
				break;
			case 'n':
				showplayer=false;
				break;
//			case 'e':
//				saveoptions=0;
//				break;
			case 'W':
				if (optarg!=NULL){
				  windowID=atoi(optarg);
				}
			case 's':
				saveoptions=val_opt(optarg);
				break;
#ifdef HAVE_KDE_GUI
			case 'k':
				gui=KDEGUI;
				break;
#endif
#ifdef HAVE_GNOME_GUI
			case 'g':
				gui=GNOMEGUI;
				break;
#endif
#ifdef HAVE_EQDE_GUI
			case 'e':
				gui=EQDEGUI;
				break;
#endif

			case 'h':
				printversion();
				printusage();
				exit(0);
			case 'V':
				printversion();
				exit(0);

			}
	}
	

  if (gui==NOGUI){
// not implemented
//    return nogui(argc,argv,fullscreen,volume,autoclose,loop,keepresolution,saveoptions,windowID);
  }
#ifdef HAVE_KDE_GUI
  else if (gui==KDEGUI){		
    return kdegui(argc,argv,fullscreen,volume,autoclose,loop,keepresolution,saveoptions,windowID);
  }
#endif

#ifdef HAVE_GNOME_GUI
  else if (gui==GNOMEGUI){
    return gnomegui(argc, argv,fullscreen,volume,autoclose,loop,keepresolution,saveoptions,windowID);
  }
#endif
#ifdef HAVE_EQDE_GUI
  else if (gui==EQDEGUI){		
    return eqdegui(argc,argv,fullscreen,volume,autoclose,loop,keepresolution,saveoptions,windowID);
  }
#endif

}

