//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//
//  Author  : Nicolas Vignal
//  E-Mail  : nicolas.vignal@fnac.net
//  Home    : http://zzplayer.sourceforge.net

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#ifndef PLUGIN_H
#define PLUGIN_H


#ifdef ZZPLAYER_DEBUG
#include <iostream>
#define DEBUGMSG(arg...){ cout << arg << endl; }
#else
#define DEBUGMSG(arg...)
#endif

typedef enum {
  STREAM_ERROR=-1,
  STREAM_STOPPED,
  STREAM_PLAYING
} Streamstatus;

// Event enumerations
enum {
  ZZ_NOEVENT=0,       // should never happen
  ZZ_VIDEORESIZE,     // no data
  ZZ_MOUSEBUTTONDOWN, // data is an char with the button number : see mouse button
  ZZ_QUIT,            // no data
  ZZ_SOMETHINGELSE    // no data
};

// mouse button
enum {
  MOUSEBUTTON_LEFT=1,
  MOUSEBUTTON_MIDDLE,
  MOUSEBUTTON_RIGHT,
  MOUSEBUTTON_SCROLLUP,
  MOUSEBUTTON_SCROLLDOWN
};

// sound info
enum {
	STREAM_WITHOUT_SOUND=0,
	STREAM_WITH_SOUND,
	NO_SOUND_SUPPORT
};

typedef struct {
  int type;
  void *data;
} ZZ_EventStruct;

typedef struct {
  int has_audio;
  int has_video;
  int width;
  int height;
  char audio_string[80];
  unsigned int total_size;
  double total_time;
} StreamInfo;


class Plugin
{

public :

  virtual char *error(void)=0;
  virtual void fullScreen(int state)=0;
  virtual void getFullInfo(StreamInfo *streamInfo)=0;
  virtual void getInfo(int *seek, double *fps, int *frame)=0;
  virtual int hasSound(void)=0;
  virtual void pause(void)=0;
  virtual int status(void)=0;
  virtual void setBilinearFilter(int bilinearfilter)=0;
  virtual void setKeepThisResolution(int keepthisresolution)=0;
  virtual void setKeepRatio(int keepratio)=0;
  virtual void setSeek(int seek)=0;
  virtual void setVolume(int vol)=0;

           Plugin(){}
  virtual ~Plugin(){}
};


#endif
