//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//
//	Requires the Qt widget libraries, available at no cost at
//      http://www.troll.no
//	
//  Author  : Nicolas Vignal
//  E-Mail  : nicolas.vignal@fnac.net
//  Home    : http://zzplayer.sourceforge.net

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#ifndef PLUGINMANAGER_CPP
#define PLUGINMANAGER_CPP

#include <dirent.h>
#include <string.h>
#include <sys/stat.h>

#include "PluginManager.h"
#include "smpeg/MpegFile.h"
#include "smpeg/MpegVCD.h"

PluginManager::PluginManager()
{
  DEBUGMSG("PluginManager::PluginManger()");
  loadPlugins();
}

PluginManager::~PluginManager()
{
  DEBUGMSG("PluginManager::~PluginManger()");


}


// try to open the file with the good plugin
// return NULL if it cannot find the right plugin
Plugin *PluginManager::openStream(const char *stream, int withaudio, int withvideo, int windowID,int (*callback)(ZZ_EventStruct *))
{
  DEBUGMSG("Plugin *PluginManager::loadStream(const char *stream, int withaudio, int withvideo, int windowID, void *(*callback)(ZZ_EventStruct *))");

  Plugin *genstream;
  genstream=NULL;

  if (strstr(stream,"vcd:")!=NULL){
    genstream = (Plugin *)new MpegVCD(stream+4,withaudio, withvideo,  windowID, (*callback));
  }
  else {
    genstream = (Plugin *)new MpegFile(stream,withaudio, withvideo, windowID, (*callback));
  }
  return genstream;
}

void PluginManager::closeStream(Plugin *stream)
{
  DEBUGMSG("PluginManager::closeStream(Plugin *stream)");
  delete stream;
}

void PluginManager::loadPlugins(void)
{
	DEBUGMSG("PluginManager::loadPlugins()");
	searchPlugins();
}



void PluginManager::searchPlugins(void)
{
  DEBUGMSG("PluginManager::searchPlugins(void)");

  char *filename, *ext;
  DIR *dir;
  int dirlen=0;
  struct dirent *ent;
  struct stat statbuf;

  dir = opendir(PLUGINS_PATH);
  if (dir){
		dirlen = strlen(PLUGINS_PATH);
		while ((ent = readdir(dir)) != NULL){
		  filename = new char[dirlen + strlen(ent->d_name) + 2];
			sprintf(filename, PLUGINS_PATH"/%s", ent->d_name);
//			DEBUGMSG("Found file : " << filename);
			if (!stat(filename, &statbuf)){
				if (S_ISREG(statbuf.st_mode)){
					if ((ext = strrchr(ent->d_name, '.')) != NULL){
						if (!strcmp(ext, ".so")){
							DEBUGMSG("Found Plugin : " << ent->d_name);
						}
						else {
							delete filename;
						}
					}
					else {
						delete filename;
					}
				}
				else {
					delete filename;
				}
			}
			else {
				delete filename;
			}
		}
	}
	else {
		DEBUGMSG("Cannot open plugins directory, opendir return : " << dir);
	}
}

#endif
