//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//
//  Author  : Nicolas Vignal
//  E-Mail  : nicolas.vignal@fnac.net
//  Home    : http://zzplayer.sourceforge.net

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#include "MpegFile.h"

// Constructor
MpegFile::MpegFile(const char *file, int withaudio, int withvideo, int windowID,int (*eventcallback)(ZZ_EventStruct *)) : VirtualMpeg(withaudio,withvideo,windowID,(*eventcallback))
{
  DEBUGMSG("MpegFile::MpegFile(const char file*, int withaudio, int withvideo, int windowID,int (*eventcallback)(ZZ_EventStruct *))")
	
  mpeg= SMPEG_new(file,&info,SDLaudio);
	
  if ( SMPEG_error(mpeg) ) {
  	return;
  }

  SMPEG_enableaudio(mpeg, SDLaudio);
  SMPEG_enablevideo(mpeg, SDLvideo);

  initVideo(0);

  setTitle(file);

  // do getinfo to fill current_offset
  SMPEG_getinfo(mpeg,&info);

  startOffset=info.current_offset;
  totalSize=info.total_size-startOffset;
 	
  // for little file totalSize could equal 0
  // I think, it's a bug in smpeg
  if (!totalSize){
    totalSize=1;
  }

  SMPEG_play(mpeg);

  if(info.has_video){
    startEventThread();
  }
}



// Destructor
MpegFile::~MpegFile()
{
  DEBUGMSG("MpegFile::~MpegFile()")
  if (mpeg) {
    if(thread!=0){
      killEventThread();
    }
    SMPEG_stop(mpeg);
    SMPEG_delete(mpeg);
    mpeg=0;
 }
}


	
// pause
void MpegFile::pause()
{
  DEBUGMSG("MpegFile::pause()");

  lockVideo();

  if(!SMPEG_error(mpeg)){
    SMPEG_pause( mpeg );
  }

  unlockVideo();
}

	
// seek a % of the mpeg file
void MpegFile::setSeek(int seek)
{
  DEBUGMSG("MpegFile::setseek(int seek)");

  lockVideo();

  if (!SMPEG_error(mpeg)) {
    if (seek==100){
      SMPEG_stop(mpeg);
    }
    else{	
      SMPEG_seek(mpeg, seek*totalSize/100+startOffset);
    }
  }
  unlockVideo();
}



// return the file position in % and the current frame and fps
void MpegFile::getInfo( int * seek, double *fps, int *frame)
{
  DEBUGMSG("void MpegFile::getInfo( int * seek, double *fps, int *frame)");
  *seek=0;
  *frame=0;
  *fps=0;

  lockVideo();

  if (!SMPEG_error(mpeg)) {
    SMPEG_getinfo(mpeg, &info);
    *seek = (100*(long long)info.current_offset-100*startOffset)/totalSize;
    *frame=info.current_frame;
    if(SMPEG_status(mpeg) == SMPEG_PLAYING){
      *fps=info.current_fps;
    }
  }
  unlockVideo();
}
