//============================================================================
//
//	ZZplayer : MPEG-I video player
// 	Copyright (C) 1999 Nicolas Vignal <nicolas.vignal@fnac.net
//
//  Author  : Nicolas Vignal
//  E-Mail  : nicolas.vignal@fnac.net
//  Home    : http://zzplayer.sourceforge.net

// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
//
//============================================================================

#include "VirtualMpeg.h"

#include <stdlib.h>
#include <X11/Xlib.h>

#ifdef XFREE86_VM
#include <X11/extensions/xf86vmode.h>
#endif

int eventThread(VirtualMpeg *object)
{
  DEBUGMSG("eventThread started");

  while (!object->checkVideoEvents()){
  	SDL_Delay(500);
  }

  DEBUGMSG("eventThread ended")

  return 0;
}

void Update(SDL_Surface *screen, int  x, int y, unsigned int w, unsigned int h)
{
  if ( screen->flags & SDL_DOUBLEBUF ) {
    SDL_Flip(screen);
  }
}


// Constructor
VirtualMpeg::VirtualMpeg( int withaudio, int withvideo, int windowID, int (*eventcallback)(ZZ_EventStruct *))
{
  sdl_screen = 0;
  mpeg = 0;
  quit = 0;
  keepThisResolution=0;
  keepRatio=1;

  char value[32];
  char buf[32];

  DEBUGMSG("VirtualMpeg::VirtualMpeg(int withaudio = " << withaudio << ", int withvideo = " << withvideo << ", int windowID= " << windowID << ",int (*eventcallback)(ZZ_EventStruct *))");

  SDLvideo=0;
  SDLaudio=0;	

  if( windowID == 0 ){
    if (putenv("SDL_WINDOWID")==-1){
      fprintf(stderr, "Cannot unset the SDL_WINDOWID variable\n");
    }
  }
  else{
#ifdef __linux
    sprintf(value,"%d", windowID);
    if (setenv("SDL_WINDOWID",value,1)==-1){
#else
    // This should work on posix sytem, but it is not working on linux if
    // I try to export the video into a netscape window with plugger
    // I don't know why. If someone has an anwser, please mail me at nicolas.vignal@fnac.net
    sprintf(value,"SDL_WINDOWID=%d\n", windowID);
    if (putenv(value)==-1){
#endif
      fprintf(stderr, "Cannot set the SDL_WINDOWID variable\n");
    }
    else {
      DEBUGMSG("Send video into window :" << windowID);
    }
  }

  if (withvideo){
    // Init SDL Video
    if (( SDL_Init( SDL_INIT_VIDEO | SDL_INIT_NOPARACHUTE ) <0 ) || !SDL_VideoDriverName(buf, 1)){
      fprintf( stderr, "Couldn't initialize SDL video: %s\n", SDL_GetError() );
      fprintf(stderr, "Will ignore video stream\n");
    }
    else {
      SDLvideo=1;
      // set the window icon
      SDL_Surface *icon=NULL;
      SDL_RWops *src = SDL_RWFromFile(PIXMAP_PATH"/video.xpm", "rb");

      if( src != NULL ){
			  icon=IMG_LoadXPM_RW(src);
			  if ( icon != NULL ) {
				  SDL_WM_SetIcon(icon, NULL);
			  }
      }
    }
  }
	
  DEBUGMSG("SDLvideo : " << SDLvideo );

#ifdef HAVE_ESOUND
  // try to force the esound driver
  if (putenv("SDL_AUDIODRIVER=esd")==-1){
    fprintf(stderr, "Cannot set the SDL_AUDIODRIVER variable to esd\n");
  }
 	// In some esd.conf there is spawn_wait_ms=100
 	// We wait 110 ms to let time to esd to respawn
 	SDL_Delay(110);
#endif

  if (withaudio){
    // Init SDL audio
    if (( SDL_Init( SDL_INIT_AUDIO | SDL_INIT_NOPARACHUTE ) < 0 ) || !SDL_VideoDriverName(buf, 1)){
      fprintf( stderr, "Couldn't initialize SDL audio: %s\n", SDL_GetError() );
      fprintf(stderr, "Will ignore audio stream\n");
    }
    else {
      SDLaudio=1;
    }
  }

#ifdef HAVE_ESOUND
  if (putenv("SDL_AUDIODRIVER")==-1){
    fprintf(stderr, "Cannot unset the SDL_AUDIODRIVER variable\n");
  }
#endif

	
  DEBUGMSG("SDLaudio : " << SDLaudio );
	
  SDL_Init( SDL_INIT_EVENTTHREAD | SDL_INIT_NOPARACHUTE);

  eventCallBack=eventcallback;
  thread=0;
  lockScreen=SDL_CreateSemaphore(1);

  if (lockScreen==NULL){
     fprintf(stderr, "Couldn't create the semaphore\n");
  }

  // dirty
  lastwidth = 0;
  lastheight = 0;	
}

// Destructor
VirtualMpeg::~VirtualMpeg()
{
  DEBUGMSG("VirtualMpeg::~VirtualMpeg()");

  if (sdl_screen){
    SDL_FreeSurface(sdl_screen);
    sdl_screen=0;
  }
  SDL_DestroySemaphore(lockScreen);
  SDL_Quit();
}

void VirtualMpeg::initVideo(int fullscreen){
	
  DEBUGMSG("initvideo( int fullscreen = " << fullscreen <<")");
	
  int xsize;
  int ysize;
  int depth;
  int ratiox;
  int ratioy;
	
  if ( info.has_video && SDLvideo ) {

    // Set up video display if needed
    const SDL_VideoInfo *video_info;
    Uint32 video_flags;
    int video_bpp;

    // Get the "native" video mode
    video_info = SDL_GetVideoInfo();
    switch (video_info->vfmt->BitsPerPixel) {
      case 16:
      case 32:
        video_bpp = video_info->vfmt->BitsPerPixel;
        break;
      default:
        video_bpp = 16;
        break;
    }

    bool paused = false;

    // lock video
    SDL_SemWait(lockScreen);

    if(SMPEG_status(mpeg) == SMPEG_PLAYING){
      SMPEG_pause(mpeg);
      paused = true;
    }
	
    SMPEG_move(mpeg,0,0);
    SMPEG_scaleXY(mpeg, 0, 0);

    if (fullscreen){
      video_flags = SDL_FULLSCREEN|SDL_DOUBLEBUF|SDL_HWSURFACE;
      DEBUGMSG("video width = "<<info.width << "  video height = " << info.height);
      // get the desktop resolution
      getResolution(&xsize,&ysize,&depth);
		
      if (!keepThisResolution){
        sdl_screen = SDL_SetVideoMode(info.width, info.height, video_bpp, video_flags);
        getResolution(&xsize,&ysize,&depth);
      }
      sdl_screen = SDL_SetVideoMode(xsize, ysize, depth, video_flags);
		
      // Hide the mouse
      SDL_ShowCursor(0);
      lastwidth=xsize;
      lastheight=ysize;
    }
    else {	
      video_flags = SDL_SWSURFACE | SDL_ASYNCBLIT | SDL_RESIZABLE;
      sdl_screen = SDL_SetVideoMode(info.width, info.height, video_bpp, video_flags);
      // Show the mouse
      SDL_ShowCursor(1);
    }

    if ( sdl_screen == NULL ) {
      fprintf(stderr, "Unable to set %dx%d video mode: %s\n",info.width, info.height, SDL_GetError());
      return;
    }


    SMPEG_setdisplay(mpeg, sdl_screen, NULL,Update);
	
    // adjust the mpeg size to the screen size, keep the ratio
    if (fullscreen){
      ratioy=xsize*info.height/info.width;
      ratiox=ysize*info.width/info.height;
	  	
      if ( ratioy <= ysize){
        SMPEG_scaleXY(mpeg, xsize, ratioy);
        SMPEG_move(mpeg,0,(ysize-ratioy)/2);
      }
      else {
        SMPEG_scaleXY(mpeg, ratiox, ysize);
        SMPEG_move(mpeg,(xsize-ratiox)/2,0);
      }
    }
    else{
      SMPEG_scaleXY(mpeg, sdl_screen->w, sdl_screen->h);
    }
	
	
    if(paused){
      SMPEG_pause(mpeg);
    }

    // free video lock
    SDL_SemPost(lockScreen);
  }
}


// check if the mpeg had sound
int VirtualMpeg::hasSound()
{
  int sound=NO_SOUND_SUPPORT;
	
  DEBUGMSG("VirtualMpeg::hasSound()");
  // lock video
  SDL_SemWait(lockScreen);

  if (SDLaudio && (!SMPEG_error(mpeg))){
    if (info.has_audio){
    	sound = STREAM_WITH_SOUND;
    }
    else {
    	sound = STREAM_WITHOUT_SOUND;
    }
  }

  // free video lock
  SDL_SemPost(lockScreen);

  return sound;
}


// set the title
void VirtualMpeg::setTitle(const char * title)
{
  DEBUGMSG("VirtualMpeg::setTitle(const char *title)");

  // lock video
  SDL_SemWait(lockScreen);

  if ( info.has_video && SDLvideo ) {
    SDL_WM_SetCaption(title,"ZZplayer Video");
  }

  // free video lock
  SDL_SemPost(lockScreen);
}

void VirtualMpeg::setVolume(int volume)
{
  DEBUGMSG("VirtualMpeg::setvolume(int vol)");
  // lock video
  SDL_SemWait(lockScreen);

  if (!SMPEG_error(mpeg)) {
    SMPEG_setvolume(mpeg, volume);
  }
  // free video lock
  SDL_SemPost(lockScreen);
}


void VirtualMpeg::clearScreen()
{
  if ( sdl_screen ) {
    SDL_FillRect(sdl_screen,NULL,SDL_MapRGB(sdl_screen->format,0,0,0));
    SDL_UpdateRect(sdl_screen, 0, 0, 0, 0);
  }
}

void VirtualMpeg::fullScreen(int state)
{
  DEBUGMSG("VirtualMpeg::fullScreen(int state)");

  if (!SMPEG_error(mpeg)){
    if ( info.has_video && SDLvideo ) {
      initVideo(state);
    }
  }
}


int VirtualMpeg::status()
{
  int status = STREAM_ERROR;
  int smpegStatus=SMPEG_ERROR;
  DEBUGMSG("VirtualMpeg::status()");

  // lock video
  SDL_SemWait(lockScreen);
	
  if (!SMPEG_error(mpeg)){
    smpegStatus = SMPEG_status(mpeg);
    if (smpegStatus==SMPEG_PLAYING){
      status=STREAM_PLAYING;
    }
    else if (smpegStatus==SMPEG_STOPPED){
      status=STREAM_STOPPED;
    }
  }

  // free video lock
  SDL_SemPost(lockScreen);

  return status;
}



// check SDL Events
int VirtualMpeg::checkVideoEvents()
{
  SDL_Event event;
  ZZ_EventStruct eventFound;

  eventFound.type=ZZ_NOEVENT;
  eventFound.data=NULL;
  						
  while (SDL_PollEvent(&event) && (mpeg!= 0) && (SMPEG_error(mpeg)==NULL) && (quit==0)){
	
    bool paused = false;
    SDL_Surface *old_screen;

    if (event.type==SDL_VIDEORESIZE){
      DEBUGMSG("Resize event w = " << event.resize.w << " h = " << event.resize.h)
      eventFound.type=ZZ_VIDEORESIZE;
      if (event.resize.w== lastwidth && event.resize.h== lastheight){
	      DEBUGMSG("Ignore resize event");
      }
      else {
        old_screen = sdl_screen;

        // lock video
        SDL_SemWait(lockScreen);

        if(SMPEG_status(mpeg) == SMPEG_PLAYING){
	        SMPEG_pause(mpeg);
	        paused = true;
        }

  			if (keepRatio){
				// check if you moved faster in x than y
					if ( abs(event.resize.w-sdl_screen->w) > abs(event.resize.h-sdl_screen->h)){
						sdl_screen = SDL_SetVideoMode(event.resize.w, (event.resize.w*info.height)/info.width, sdl_screen->format->BitsPerPixel, sdl_screen->flags);
					}
					else{
						sdl_screen = SDL_SetVideoMode((event.resize.h*info.width)/info.height, event.resize.h, sdl_screen->format->BitsPerPixel, sdl_screen->flags);
					}
				}
				else {
					sdl_screen = SDL_SetVideoMode(event.resize.w, event.resize.h, sdl_screen->format->BitsPerPixel, sdl_screen->flags);
				}

				if ( old_screen != sdl_screen ){
					SMPEG_setdisplay(mpeg, sdl_screen, NULL, Update);
				}


        SMPEG_scaleXY(mpeg, sdl_screen->w, sdl_screen->h);

        if(paused){
	        SMPEG_pause(mpeg);
        }

        // free video lock
        SDL_SemPost(lockScreen);
      }
	  }
    else if (event.type==SDL_MOUSEBUTTONDOWN){
      eventFound.type=ZZ_MOUSEBUTTONDOWN;
      eventFound.data=(void *)event.button.button;
    }
	  else if (event.type==SDL_QUIT){
      DEBUGMSG("Quit event")
      quit=1;
      eventFound.type=ZZ_QUIT;
    }
    else {
      eventFound.type=ZZ_SOMETHINGELSE;
      DEBUGMSG("other kind of event");
    }

  	// launch the callback function of the GUI
  	if  (eventCallBack && (eventFound.type!=ZZ_SOMETHINGELSE)){
    	(*eventCallBack)(&eventFound);
  	}
  }
  
  return quit;
}


// get current resolution
void VirtualMpeg::getResolution( int *xsize, int *ysize, int *depth)
{
	Display *dpy=NULL;
	int scr = 0;
#ifdef XFREE86_VM
	XF86VidModeModeLine mode_line;
#endif
	int dot_clock;
	
	dpy = XOpenDisplay (NULL);
	if (!dpy) {
		fprintf (stderr, "unable to open display \n");
		exit (1);
    	}

#ifdef XFREE86_VM
	if (!XF86VidModeGetModeLine (dpy, scr, &dot_clock, &mode_line)){	
		fprintf(stderr, "unable to get the current resolution\n");
		exit(1);
	}
	*xsize=mode_line.hdisplay;
	*ysize=mode_line.vdisplay;	
#else
	{
		SDL_Rect **modes;
		modes = SDL_ListModes(NULL, SDL_FULLSCREEN | SDL_HWSURFACE);

		if (modes == (SDL_Rect **)0) {
			fprintf(stderr, "No modes available\n");
			goto hack;
		} else if (modes == (SDL_Rect **)-1) {
			fprintf(stderr, "All resolutions available\n");
			/* XXX what the hell does that mean? */
			goto hack;
		} else {
			int i;
			fprintf(stderr, "available modes:\n");
			/* XXX does this always return the best one first? */
			*xsize = modes[0]->w;
			*ysize = modes[0]->h;
			for (i=0; modes[i]; i++)
				fprintf(stderr,
					" %d x %d\n", modes[i]->w, modes[i]->h);
			goto out;
			
		}
	}
hack:
	*xsize = 640;
	*ysize = 480;
out:
#endif

	*depth=XDefaultDepth(dpy,scr);

					    	
	DEBUGMSG("Current resolution : x = " << *xsize << " y = "<< *ysize << " z = "<< *depth);
}

// test if error
char *VirtualMpeg::error(void)
{
	DEBUGMSG("char *VirtualMpeg::error(void)");
	return SMPEG_error(mpeg);
}



// set keepthisresolution
void VirtualMpeg::setKeepThisResolution(int keepthisresolution)
{
	DEBUGMSG("void VirtualMpeg::setKeepThisResolution(int keepthisresolution)");
	keepThisResolution=keepthisresolution;
}


// set keepthisresolution
void VirtualMpeg::setKeepRatio(int keepratio)
{
	DEBUGMSG("void VirtualMpeg::setKeepRatio(int keepratio)");
	keepRatio=keepratio;
}


// Get full info about the stream
void VirtualMpeg::getFullInfo(StreamInfo *streamInfo){
  DEBUGMSG("void VirtualMpeg::getFullInfo(streamInfo *streamInfo)");

  // lock video
  SDL_SemWait(lockScreen);

  if (!SMPEG_error(mpeg)){
    streamInfo->has_audio    = info.has_audio;
    streamInfo->has_video    = info.has_video;
    streamInfo->width        = info.width;
    streamInfo->height       = info.height;
    streamInfo->audio_string = info.audio_string;
    streamInfo->total_size   = info.total_size;
    streamInfo->total_time   = info.total_time;
  }
  // free video lock
  SDL_SemPost(lockScreen);
}


// Enable or disable the bilinear filter
void VirtualMpeg::setBilinearFilter(int bilinearfilter){
  DEBUGMSG("void VirtualMpeg::setBilinearFilter(int bilinearfilter)");

  SMPEG_Filter * filter;

  // lock video
  SDL_SemWait(lockScreen);

  if ( info.has_video && SDLvideo ) {

    if(bilinearfilter){
      // Activate the bilinear filter
      filter = SMPEGfilter_bilinear();
      filter = SMPEG_filter( mpeg, filter );
      filter->destroy(filter);
    }
    else {
      // Reset to default (null) filter
      filter = SMPEGfilter_null();
      filter = SMPEG_filter( mpeg, filter );
      filter->destroy(filter);
    }
  }

  // free video lock
  SDL_SemPost(lockScreen);
}

// start EventThread, only if we use the video
void VirtualMpeg::startEventThread(void){
  DEBUGMSG("void VirtualMpeg::startEventThread(void)");
  thread=SDL_CreateThread((int(*)(void *))eventThread, this);
}



// kill EventThread
void VirtualMpeg::killEventThread(void){
  DEBUGMSG("void VirtualMpeg::killEventThread(void)");

  quit=1;
  SDL_WaitThread(thread, NULL);
}

void VirtualMpeg::lockVideo(void){
  // lock video
  SDL_SemWait(lockScreen);
}

void VirtualMpeg::unlockVideo(void){
  // free video lock
  SDL_SemPost(lockScreen);
}


